PROGRAM FUND29;

(*  Climate Framework for Uncertainty, Negotiation and Distribution       *)
(*  Version 2.90  -  10 December 2005                                     *)
(*  Richard S.J. Tol                                                      *)
(*  Hamburg, Vrije and Carnegie Mellon Universities                       *)

USES
    Crt, RP290, GenRand;

CONST
     scratch = true;                  (* start with no measures ? *)
     CBA = false;                      (* cost-benefit analysis ? *)
     CEA = false;                     (* cost-effectiveness analysis ? *)
     cooperate = false;               (* cooperate ? *)
     repeated = false;
     uncertain = false;               (* uncertainty ? *)
     scengmt = false;                 (* scenario global mean temperature ? *)
     kantian = false;
     compens = false;
     aid = false;
     ppp = 0;
     MargCO2 = true;
     MargCH4 = false;
     MargN2O = false;
     MargSF6 = false;

     consleak = 0.25;                 (* consumption leak *)
(*     perm = 0.5;                      (* share permanent emission reduction *)
     knowpar = 0.9;
     knowgpar = 0.1;
     compth = 100;                    (* compensation threshold *)

     startyear = 62;
     delaybric = 0;  (*18*)
     delayrow = 0;   (*38*)
     transition = 0; (*20*)

     startper = 1;

     riskave = 1;
     ineqave = 0;

     ampl = 4;
     altruism = 0.00; (* altruism should be zero if cooperation *)

     runs = 0;

     bforst = 1.00;
     btaxst = 0.0001;
     bch4st = 0.0001;
     bn2ost = 0.0001;

     CBaU = 1792.3352;
     CMax = 550;

     gwpch4 = 21*22/6/1000;
     gwpn2o = 310*7/6/1000;

     geoth = 1000;

TYPE
    measure = regarr;
    matmeas = ARRAY [1..NoReg,1..NOI] OF double;
    regper = ARRAY [1..NoReg,1..NoPer] OF double;
    boolmea = ARRAY [1..NoReg,1..NOI] OF boolean;

VAR
   forstep, taxstep, ch4step, n2ostep, perm: double;
   c2000, c2025, c2050, c2075, c2100, c2125, c2150, c2175, c2200, c2225, c2250, c2275, c2300: double;
   toptper,globemm,globcum,globknow,minint,bco2stock: double;
   optper, mdp: word;
   globviol: boolean;
   emmrec, count: regarr;
   coop: regper;

   spbm, spbmbg, spbmsd: double;
   vsl, vslbg, vslsd, vyll, vyllbg, vyllsd, vmorb, vmorbbg, vmorbsd, valincbg, valincsd, valbase: double;
   watechrate, wrel, wrelbg, wrelsd, wrnl, wrnlbg, wrnlsd: double;
   agel, agelbg, agelsd, agnl, agnlbg, agnlsd, forel, forelbg, forelsd, fornl, fornlbg, fornlsd: double;
   heel, heelbg, heelsd, henl, henlbg, henlsd: double;
   ceel, ceelbg, ceelsd, cenl, cenlbg, cenlsd: double;
   dfnl, dfnlbg, dfnlsd: double;
   smnl, smnlbg, smnlsd: double;
   malnl, malnlbg, malnlsd: double;
   diamortel, diamortelbg, diamortelsd, diayldel, diayldelbg, diayldelsd: double;
   diamortnl, diamortnlbg, diamortnlsd, diayldnl, diayldnlbg, diayldnlsd: double;
   bioloss, biosens, bioshare, nospecies: double;
   dvbmbg, dvbmsd, drydelbg, drydelsd, incdens, wvbmbg, wvbmsd, wetdelbg, wetdelsd: double;
   immcst, immcstbg, immcstsd, entertime, entertimebg, entertimesd: double;
   emcst, emcstbg, emcstsd, leavedel, leavedelbg, leavedelsd: double;
   plusel, pluselbg, pluselsd, cvlin, cvlinbg, cvlinsd, rlin, rlinbg, rlinsd: double;
   maxcardvasc, maxcardvascbg, maxcardvascsd: double;
   addmp, vbel, vbelbg, vbelsd: double;
   forco2, forco2bg,forco2sd: double;
   MC, maxRF, RF2100, RF2300, maxPCO2, PCO2100, maxyr, T2100: double;
   MCA: ARRAY [0..10, 0..NoReg] OF double;
   so2pop, so2inc, so2carb, so2dir, so2ind: double;
   sf6gdp, sf6ypc: double;

FUNCTION Interact(M,N: double): double;
(* interaction radiative forcing methane and nitrous oxide (in ppbv) *)

VAR hulp: double;

BEGIN
     hulp:= 1 + pow(M*N,0.75)*2.01E-5 + pow(M*N,1.52)*M*5.31E-15;
     interact:= 0.47*log(hulp)
END;

PROCEDURE InitImpactPar;

(* CODE
   sp = species
   for = forestry
   ag = agriculture
   he = heating energy
   ce = cooling energy
   wr = water resources
   df = dengue fever
   mal = malaria
   sm = schistosomiasis
   ccp = cardiovascular cold 65 plus
   ccm = cardiovascular cold 65 minus
   chp = cardiovascular heat 65 plus
   chm = cardiovascular heat 65 minus
   rp = respiratory 65 plus
   rm = respiratory 65 minus

   bg = best guess
   sd = standard deviation
   bm = benchmark
   nl = non-linearity
   el = elasticity
   del = delay
   l = linear
   q = quadratic
*)

VAR i,j: word;

BEGIN
     bioloss:= biolossbg;
     biosens:= biosensbg;
     bioshare:= biosharebg;
     wrel:= wrelbg;
     wrnl:= wrnlbg;
     watechrate:= watechratebg;
     agel:= agelbg;
     agnl:= agnlbg;
     forco2:= forco2bg;
     forel:= forelbg;
     fornl:= fornlbg;
     malnl:= malnlbg;
     dfnl:= dfnlbg;
     smnl:= smnlbg;
     heel:= heelbg;
     henl:= henlbg;
     ceel:= ceelbg;
     cenl:= cenlbg;
     plusel:= pluselbg;
     vsl:= vslbg;
     vyll:= vyllbg;
     vmorb:= vmorbbg;
     emcst:= emcstbg;
     immcst:= immcstbg;
     cvlin:= cvlinbg;
     rlin:= rlinbg;
     diamortel:= diamortelbg;
     diamortnl:= diamortnlbg;
     spbm:= spbmbg;
     maxcardvasc:= maxcardvascbg;
     leavedel:= leavedelbg;
     entertime:= entertimebg;

     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO BEGIN
              valinc:= valincbg;
              valadj:= valbase/valinc/(1+valbase/valinc);
              wrbm:= wrbmbg;
              forbm:= forbmbg;
              hebm:= hebmbg;
              cebm:= cebmbg;
              dfch:= dfchbg;
              smch:= smchbg;
              malch:= malchbg;
              diayldel:= diayldelbg;
              diayldnl:= diayldnlbg;
              agrbm:= agrbmbg;
              aglbm:= aglbmbg;
              agopt:= agoptbg;
              aglparl:= -2*agopt*aglbm/(10.24-6.4*agopt);
              aglparq:= aglbm/(10.24-6.4*agopt);
              IF aglparq > 0 THEN BEGIN
                 aglparq:= 0;
                 aglparl:= aglbm/3.2
              END;
              agcbm:= agcbmbg/ln(2);
              agtime:= agtimebg;
              ccplbm:= ccplbmbg;
              ccpqbm:= ccpqbmbg;
              ccmlbm:= ccmlbmbg;
              ccmqbm:= ccmqbmbg;
              chplbm:= chplbmbg;
              chpqbm:= chpqbmbg;
              chmlbm:= chmlbmbg;
              chmqbm:= chmqbmbg;
              vbel:= vbelbg;
              rlbm:= rlbmbg;
              rqbm:= rqbmbg;
              wlbm:= wlbmbg;
              wvbm:= wvbmbg;
              wetdel:= wetdelbg;
              wmbm:= wmbmbg;
              dlbm:= dlbmbg;
              dvbm:= dvbmbg;
              drydel:= drydelbg;
              pc:= pcbg;
              FOR j:= 1 TO NoReg DO
                  immigrate[j]:= immigratebg[j]
         END
END;

PROCEDURE InitCoalition;

VAR i,j: word;

BEGIN
       coop[1,1]:= 0;   coop[1,2]:= 0;   coop[1,3]:= 0;   coop[1,4]:= 0;
       coop[1,5]:= 0;   coop[1,6]:= 0;   coop[1,7]:= 0;   coop[1,8]:= 0;
       coop[1,9]:= 0;  coop[1,10]:= 0;  coop[1,11]:= 0;  coop[1,12]:= 0;
      coop[1,13]:= 0;  coop[1,14]:= 0;  coop[1,15]:= 0;  coop[1,16]:= 0;
      coop[1,17]:= 0;  coop[1,18]:= 0;

       coop[2,1]:= 0;   coop[2,2]:= 0;   coop[2,3]:= 0;   coop[2,4]:= 0;
       coop[2,5]:= 1;   coop[2,6]:= 1;   coop[2,7]:= 1;   coop[2,8]:= 1;
       coop[2,9]:= 1;  coop[2,10]:= 1;  coop[2,11]:= 1;  coop[2,12]:= 1;
      coop[2,13]:= 1;  coop[2,14]:= 1;  coop[2,15]:= 1;  coop[2,16]:= 1;
      coop[2,17]:= 1;  coop[2,18]:= 1;

       coop[3,1]:= 0;   coop[3,2]:= 0;   coop[3,3]:= 0;   coop[3,4]:= 0;
       coop[3,5]:= 0;   coop[3,6]:= 0;   coop[3,7]:= 0;   coop[3,8]:= 0;
       coop[3,9]:= 0;  coop[3,10]:= 0;  coop[3,11]:= 0;  coop[3,12]:= 0;
      coop[3,13]:= 0;  coop[3,14]:= 0;  coop[3,15]:= 0;  coop[3,16]:= 0;
      coop[3,17]:= 0;  coop[3,18]:= 0;

       coop[4,1]:= 1;   coop[4,2]:= 1;   coop[4,3]:= 1;   coop[4,4]:= 1;
       coop[4,5]:= 1;   coop[4,6]:= 1;   coop[4,7]:= 1;   coop[4,8]:= 1;
       coop[4,9]:= 1;  coop[4,10]:= 1;  coop[4,11]:= 1;  coop[4,12]:= 1;
      coop[4,13]:= 1;  coop[4,14]:= 1;  coop[4,15]:= 1;  coop[4,16]:= 1;
      coop[4,17]:= 1;  coop[4,18]:= 1;

       coop[5,1]:= 0;   coop[5,2]:= 0;   coop[5,3]:= 0;   coop[5,4]:= 0;
       coop[5,5]:= 0;   coop[5,6]:= 0;   coop[5,7]:= 0;   coop[5,8]:= 0;
       coop[5,9]:= 0;  coop[5,10]:= 0;  coop[5,11]:= 0;  coop[5,12]:= 0;
      coop[5,13]:= 0;  coop[5,14]:= 0;  coop[5,15]:= 0;  coop[5,16]:= 0;
      coop[5,17]:= 0;  coop[5,18]:= 0;

       coop[6,1]:= 0;   coop[6,2]:= 0;   coop[6,3]:= 0;   coop[6,4]:= 0;
       coop[6,5]:= 0;   coop[6,6]:= 0;   coop[6,7]:= 0;   coop[6,8]:= 0;
       coop[6,9]:= 1;  coop[6,10]:= 1;  coop[6,11]:= 1;  coop[6,12]:= 1;
      coop[6,13]:= 1;  coop[6,14]:= 1;  coop[6,15]:= 1;  coop[6,16]:= 1;
      coop[6,17]:= 1;  coop[6,18]:= 1;

       coop[7,1]:= 0;   coop[7,2]:= 0;   coop[7,3]:= 0;   coop[7,4]:= 0;
       coop[7,5]:= 0;   coop[7,6]:= 0;   coop[7,7]:= 1;   coop[7,8]:= 1;
       coop[7,9]:= 1;  coop[7,10]:= 1;  coop[7,11]:= 1;  coop[7,12]:= 1;
      coop[7,13]:= 1;  coop[7,14]:= 1;  coop[7,15]:= 1;  coop[7,16]:= 1;
      coop[7,17]:= 1;  coop[7,18]:= 1;

       coop[8,1]:= 0;   coop[8,2]:= 0;   coop[8,3]:= 0;   coop[8,4]:= 0;
       coop[8,5]:= 0;   coop[8,6]:= 0;   coop[8,7]:= 0;   coop[8,8]:= 0;
       coop[8,9]:= 0;  coop[8,10]:= 0;  coop[8,11]:= 0;  coop[8,12]:= 0;
      coop[8,13]:= 0;  coop[8,14]:= 0;  coop[8,15]:= 0;  coop[8,16]:= 0;
      coop[8,17]:= 0;  coop[8,18]:= 0;

       coop[9,1]:= 1;   coop[9,2]:= 1;   coop[9,3]:= 1;   coop[9,4]:= 1;
       coop[9,5]:= 1;   coop[9,6]:= 1;   coop[9,7]:= 1;   coop[9,8]:= 1;
       coop[9,9]:= 1;  coop[9,10]:= 1;  coop[9,11]:= 1;  coop[9,12]:= 1;
      coop[9,13]:= 1;  coop[9,14]:= 1;  coop[9,15]:= 1;  coop[9,16]:= 1;
      coop[9,17]:= 1;  coop[9,18]:= 1;

      coop[10,1]:= 0;  coop[10,2]:= 0;  coop[10,3]:= 0;  coop[10,4]:= 0;
      coop[10,5]:= 0;  coop[10,6]:= 0;  coop[10,7]:= 0;  coop[10,8]:= 0;
      coop[10,9]:= 0; coop[10,10]:= 0; coop[10,11]:= 0; coop[10,12]:= 0;
     coop[10,13]:= 0; coop[10,14]:= 0; coop[10,15]:= 0; coop[10,16]:= 0;
     coop[10,17]:= 0; coop[10,18]:= 0;

      coop[11,1]:= 1;  coop[11,2]:= 1;  coop[11,3]:= 1;  coop[11,4]:= 1;
      coop[11,5]:= 1;  coop[11,6]:= 1;  coop[11,7]:= 1;  coop[11,8]:= 1;
      coop[11,9]:= 1; coop[11,10]:= 1; coop[11,11]:= 1; coop[11,12]:= 1;
     coop[11,13]:= 1; coop[11,14]:= 1; coop[11,15]:= 1; coop[11,16]:= 1;
     coop[11,17]:= 1; coop[11,18]:= 1;

      coop[12,1]:= 0;  coop[12,2]:= 0;  coop[12,3]:= 0;  coop[12,4]:= 0;
      coop[12,5]:= 0;  coop[12,6]:= 0;  coop[12,7]:= 0;  coop[12,8]:= 0;
      coop[12,9]:= 0; coop[12,10]:= 0; coop[12,11]:= 0; coop[12,12]:= 0;
     coop[12,13]:= 0; coop[12,14]:= 0; coop[12,15]:= 0; coop[12,16]:= 0;
     coop[12,17]:= 0; coop[12,18]:= 0;

      coop[13,1]:= 0;  coop[13,2]:= 0;  coop[13,3]:= 0;  coop[13,4]:= 0;
      coop[13,5]:= 0;  coop[13,6]:= 0;  coop[13,7]:= 0;  coop[13,8]:= 0;
      coop[13,9]:= 1; coop[13,10]:= 1; coop[13,11]:= 1; coop[13,12]:= 1;
     coop[13,13]:= 1; coop[13,14]:= 1; coop[13,15]:= 1; coop[13,16]:= 1;
     coop[13,17]:= 1; coop[13,18]:= 1;

      coop[14,1]:= 0;  coop[14,2]:= 0;  coop[14,3]:= 0;  coop[14,4]:= 0;
      coop[14,5]:= 0;  coop[14,6]:= 0;  coop[14,7]:= 1;  coop[14,8]:= 1;
      coop[14,9]:= 1; coop[14,10]:= 1; coop[14,11]:= 1; coop[14,12]:= 1;
     coop[14,13]:= 1; coop[14,14]:= 1; coop[14,15]:= 1; coop[14,16]:= 1;
     coop[14,17]:= 1; coop[14,18]:= 1;

      coop[15,1]:= 0;  coop[15,2]:= 0;  coop[15,3]:= 0;  coop[15,4]:= 0;
      coop[15,5]:= 0;  coop[15,6]:= 0;  coop[15,7]:= 0;  coop[15,8]:= 0;
      coop[15,9]:= 0; coop[15,10]:= 0; coop[15,11]:= 0; coop[15,12]:= 0;
     coop[15,13]:= 0; coop[15,14]:= 0; coop[15,15]:= 0; coop[15,16]:= 0;
     coop[15,17]:= 0; coop[15,18]:= 0;

      coop[16,1]:= 1;  coop[16,2]:= 1;  coop[16,3]:= 1;  coop[16,4]:= 1;
      coop[16,5]:= 1;  coop[16,6]:= 1;  coop[16,7]:= 1;  coop[16,8]:= 1;
      coop[16,9]:= 1; coop[16,10]:= 1; coop[16,11]:= 1; coop[16,12]:= 1;
     coop[16,13]:= 1; coop[16,14]:= 1; coop[16,15]:= 1; coop[16,16]:= 1;
     coop[16,17]:= 1; coop[16,18]:= 1;

     FOR i:= 1 TO NoReg DO
         FOR j:= 1 TO NoPer DO
             coop[i,j]:= 0
END;

PROCEDURE Initialise;

VAR i, j: word;
    fil: text;

BEGIN
     addmp:= 0;
     optper:= startper;
     C2200:= CBaU;
     FOR i:= 1 TO NoReg DO
         WITH Economy[i] DO BEGIN
              oda:= 0;
              new(ypcgrowth);
              new(pgrowth);
              new(aeei);
              new(acei);
              new(ch4em);
              new(n2oem);
              new(forestemm);
              new(scentemp)
         END;
     WITH Climate DO BEGIN
          decay[1]:= 1;
          decay[2]:= exp(-1/lifeco2);
          decay[3]:= exp(-1/lifeco3);
          decay[4]:= exp(-1/lifeco4);
          decay[5]:= exp(-1/lifeco5);
          frac[1]:= 0.13;
          frac[2]:= 0.20;
          frac[3]:= 0.32;
          frac[4]:= 0.25;
          frac[5]:= 0.10;
          ch4decay:= ch4dec;
          n2odecay:= n2odec;
          sf6decay:= sf6dec;
          ch4sd:= 6.8346;
          n2osd:= 0.0059;
          sf6sd:= 0.0800; (* 1% of 2000 emissions *)
          so2sd:= 1.6000; (* 1% of 2000 emissions *)
          new(acco2);
          new(acch4);
          new(acn2o);
          new(acsf6)
     END;

     Economy[1].name:=  'USA';     Economy[2].name:=  'CAN';
     Economy[3].name:=  'WEU';     Economy[4].name:=  'JPK';
     Economy[5].name:=  'ANZ';     Economy[6].name:=  'EEU';
     Economy[7].name:=  'FSU';     Economy[8].name:=  'MDE';
     Economy[9].name:=  'CAM';     Economy[10].name:= 'SAM';
     Economy[11].name:= 'SAS';     Economy[12].name:= 'SEA';
     Economy[13].name:= 'CHI';     Economy[14].name:= 'NAF';
     Economy[15].name:= 'SSA';     Economy[16].name:= 'SIS';

     Economy[1].GDP90:=  6521;   Economy[2].GDP90:=   534;
     Economy[3].GDP90:=  8454;   Economy[4].GDP90:=  5278;
     Economy[5].GDP90:=   370;   Economy[6].GDP90:=   351;
     Economy[7].GDP90:=   779;   Economy[8].GDP90:=   535;
     Economy[9].GDP90:=   305;   Economy[10].GDP90:= 1139;
     Economy[11].GDP90:=  376;   Economy[12].GDP90:= 2136;
     Economy[13].GDP90:=  521;   Economy[14].GDP90:=  144;
     Economy[15].GDP90:=  297;   Economy[16].GDP90:=  116;

     Economy[1].pop90:=   256;   Economy[2].pop90:=    28;
     Economy[3].pop90:=   377;   Economy[4].pop90:=   166;
     Economy[5].pop90:=    20;   Economy[6].pop90:=   122;
     Economy[7].pop90:=   289;   Economy[8].pop90:=   193;
     Economy[9].pop90:=   112;   Economy[10].pop90:=  296;
     Economy[11].pop90:= 1117;   Economy[12].pop90:=  562;
     Economy[13].pop90:= 1183;   Economy[14].pop90:=  118;
     Economy[15].pop90:=  501;   Economy[16].pop90:=   38;

     Economy[1].urbcorr:=   0.103545;   Economy[2].urbcorr:=   0.059130;
     Economy[3].urbcorr:=   0.064797;   Economy[4].urbcorr:=   0.100579;
     Economy[5].urbcorr:=  -0.050885;   Economy[6].urbcorr:=   0.057221;
     Economy[7].urbcorr:=  -0.077465;   Economy[8].urbcorr:=   0.008454;
     Economy[9].urbcorr:=  -0.072193;   Economy[10].urbcorr:= -0.124256;
     Economy[11].urbcorr:=  0.172639;   Economy[12].urbcorr:=  0.679370;
     Economy[13].urbcorr:=  0.286088;   Economy[14].urbcorr:=  0.037760;
     Economy[15].urbcorr:=  0.477441;   Economy[16].urbcorr:=  0.085267;

     Economy[1].coastpd:=   1.04;     Economy[2].coastpd:=   1.07;
     Economy[3].coastpd:=   4.48;     Economy[4].coastpd:=   1.04;
     Economy[5].coastpd:=   1.27;     Economy[6].coastpd:=   3.01;
     Economy[7].coastpd:=   0.75;     Economy[8].coastpd:=   3.42;
     Economy[9].coastpd:=   0.69;     Economy[10].coastpd:=  1.97;
     Economy[11].coastpd:=  2.31;     Economy[12].coastpd:=  1.80;
     Economy[13].coastpd:=  4.46;     Economy[14].coastpd:= 17.61;
     Economy[15].coastpd:=  9.69;     Economy[16].coastpd:=  2.51;

     FOR i:= 1 TO NoReg DO BEGIN
         Economy[i].ypc90:= Economy[i].GDP90/Economy[i].pop90*1000;
         Economy[i].ypc00:= Economy[i].ypc90
     END;

     Economy[1].plus90:=  0.1239;     Economy[2].plus90:=  0.1122;
     Economy[3].plus90:=  0.1473;     Economy[4].plus90:=  0.1490;
     Economy[5].plus90:=  0.1115;     Economy[6].plus90:=  0.1034;
     Economy[7].plus90:=  0.0931;     Economy[8].plus90:=  0.0358;
     Economy[9].plus90:=  0.0384;     Economy[10].plus90:= 0.0486;
     Economy[11].plus90:= 0.0406;     Economy[12].plus90:= 0.0400;
     Economy[13].plus90:= 0.0856;     Economy[14].plus90:= 0.0373;
     Economy[15].plus90:= 0.0291;     Economy[16].plus90:= 0.0604;

     Economy[1].cardvasc90:=  0.00350;     Economy[2].cardvasc90:=  0.00350;
     Economy[3].cardvasc90:=  0.00436;     Economy[4].cardvasc90:=  0.00263;
     Economy[5].cardvasc90:=  0.00263;     Economy[6].cardvasc90:=  0.00481;
     Economy[7].cardvasc90:=  0.00874;     Economy[8].cardvasc90:=  0.00198;
     Economy[9].cardvasc90:=  0.00182;     Economy[10].cardvasc90:= 0.00182;
     Economy[11].cardvasc90:= 0.00281;     Economy[12].cardvasc90:= 0.00203;
     Economy[13].cardvasc90:= 0.00205;     Economy[14].cardvasc90:= 0.00237;
     Economy[15].cardvasc90:= 0.00149;     Economy[16].cardvasc90:= 0.00182;

     Economy[1].resp90:=  0.00053;     Economy[2].resp90:=  0.00053;
     Economy[3].resp90:=  0.00050;     Economy[4].resp90:=  0.00037;
     Economy[5].resp90:=  0.00037;     Economy[6].resp90:=  0.00030;
     Economy[7].resp90:=  0.00052;     Economy[8].resp90:=  0.00019;
     Economy[9].resp90:=  0.00039;     Economy[10].resp90:= 0.00039;
     Economy[11].resp90:= 0.00039;     Economy[12].resp90:= 0.00046;
     Economy[13].resp90:= 0.00011;     Economy[14].resp90:= 0.00037;
     Economy[15].resp90:= 0.00038;     Economy[16].resp90:= 0.00039;

     FOR i:= 1 TO NoReg DO
         FOR j:= 1 TO NoPer DO
             economy[i].trade[j]:= 0;

     ReadWater;
     ReadForestry;
     ReadHeating;
     ReadCooling;
     ReadDengue;
     ReadSchisto;
     ReadMalaria;
     ReadDiarrhea;
     ReadAgricult;
     ReadCardHeat;
     ReadCardCold;
     ReadResp;
     ReadYLL;
     ReadYLD;
     ReadWetland;
     ReadDryland;
     ReadProtect;
     ReadImmigrate;

     ReadIncome;
     ReadSulphur;
     ReadSF6;
     ReadEmissint;
     ReadPopulation;
     ReadYpCGrowth;
     ReadAgrish;
     ReadPGrowth;
     ReadACEI;
     ReadAEEI;
     ReadForestEmm;
     ReadForParam;
     ReadDiscRate;
     ReadCH4Em;
     ReadN2OEm;
     ReadBaseRegTemp;
     ReadScenGMT;
     ReadNCEmmRestr;
     ReadCEmmRestr;
     ReadScenUnc;
     ReadMethPar;
     ReadNitrOxPar;
     WITH climate DO BEGIN
          temps:= modet/5.68/ln(2);
          seas:= modes;
          delaysea:= 1/lifesea;
          delaytemp:= 1/lifetemp
     END;
     forstep:= bforst;
     taxstep:= btaxst;
     ch4step:= bch4st;
     n2ostep:= bn2ost;
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO BEGIN
              ecgradd:= 0;
              pgadd:= 0;
              aeeiadd:= 0;
              aceiadd:= 0;
              foremadd:= 0;
              taxmp:= 1;
              formp:= 1
         END;
     valbase:= 24401;
     valincbg:= 30000;
     valincsd:= 10000;
  (* 20000 is benchmark income; 15965.86 is OECD income in 1990 *)
     spbmbg:= 50;
     spbmsd:= 50;
     dvbmbg:= 4;
     dvbmsd:= 2;
     drydelbg:= 0;
     drydelsd:= 0.1;
     incdens:= 635;
     (* 635 is income density in 1990 in OECD *)
     wvbmbg:= 5;
     wvbmsd:= 2.5;
     wetdelbg:= 0;
     wetdelsd:= 0.1;
     immcstbg:= 0.4;
     immcstsd:= 0.2;
     entertimebg:= 3;
     entertimesd:= 1;
     emcstbg:= 3;
     emcstsd:= 1.5;
     leavedelbg:= 0;
     leavedelsd:= 0.1;
     wrelbg:= 0.85;
     wrelsd:= 0.15;
     wrnlbg:= 1;
     wrnlsd:= 0.5;
     forelbg:= 0.31;
     forelsd:= 0.20;
     fornlbg:= 1.0;
     fornlsd:= 0.5;
     forco2bg:=2.2972;
     forco2sd:=2.2972/2;
     heelbg:= 0.8;
     heelsd:= 0.2;
     henlbg:= 0.5;
     henlsd:= 0.5;
     ceelbg:= 0.8;
     ceelsd:= 0.2;
     cenlbg:= 1.5;
     cenlsd:= 0.5;
     dfnlbg:= 1.0;
     dfnlsd:= 0.5;
     smnlbg:= 1.0;
     smnlsd:= 0.5;
     malnlbg:= 1.0;
     malnlsd:= 0.5;
     diayldelbg:= -0.418406;
     diayldelsd:= 0.119223;
     diamortelbg:= -1.578625;
     diamortelsd:= 0.232507;
     diayldnlbg:= 0.699241;
     diayldnlsd:= 0.260643;
     diamortnlbg:= 1.141714;
     diamortnlsd:= 0.508302;
     agnlbg:= 2;
     agnlsd:= 0.5;
     agelbg:= 0.31*(1-ppp+0.7195*ppp);
     agelsd:= 0.15;
     pluselbg:= 0.25*(1-ppp+0.7195*ppp);
     pluselsd:= 0.08;
     vbelbg:= -2.65*(1-ppp) - 3.26*ppp;
     vbelsd:= 0.69*(1-ppp) + 1.05*ppp;
     cvlinbg:= 0.025901;
     cvlinsd:= 0.009574;
     rlinbg:= 0.001583;
     rlinsd:= 0.000457;
     maxcardvascbg:= 0.05;
     maxcardvascsd:= 0.02;
     vslbg:= 200;
     vslsd:= 100;
     vyllbg:= 10;
     vyllsd:= 5;
     vmorbbg:= 0.8;
     vmorbsd:= 1.2;
     so2pop:= 0.33;
     so2inc:=-0.45*(1-ppp+0.7195*ppp);
     so2carb:= 1.02;
     so2dir:= 0.03*(1-ppp+1.116*ppp);
     so2ind:= 0.08*(1-ppp+1.116*ppp);
     sf6gdp:= 0.0054083/23.9;  (* this was measured in MMT CO2eq, not in KT SF6; GWP = 23900 *)
     sf6ypc:= -0.000058961/23.9;
END;

PROCEDURE ReInit;

VAR i, j: word;

BEGIN
     ReadMethPar; (* ??? *)
     ReadNitroxPar; (* ??? *)

     economy[1].cummemm:=  22967;        economy[2].cummemm:=   1999;
     economy[3].cummemm:=  25854;        economy[4].cummemm:=   1135;
     economy[5].cummemm:=    526;        economy[6].cummemm:=   2199;
     economy[7].cummemm:=   3214;        economy[8].cummemm:=     86;
     economy[9].cummemm:=    212;        economy[10].cummemm:=   696;
     economy[11].cummemm:=   569;        economy[12].cummemm:=   225;
     economy[13].cummemm:=   775;        economy[14].cummemm:=    64;
     economy[15].cummemm:=   472;        economy[16].cummemm:=     0;

     Economy[1].area:=   9207581;       Economy[2].area:=   9872483;
     Economy[3].area:=   3705040;       Economy[4].area:=    472934;
     Economy[5].area:=   9235211;       Economy[6].area:=   2186128;
     Economy[7].area:=  21716915;       Economy[8].area:=   6313671;
     Economy[9].area:=   2440575;       Economy[10].area:= 17451759;
     Economy[11].area:=  5137567;       Economy[12].area:=  4958546;
     Economy[13].area:= 10958008;       Economy[14].area:=  3953026;
     Economy[15].area:= 24329621;       Economy[16].area:=   332534;

     globcum:= 60994.3;
     globknow:= 1;
     nospecies:= nospecbase;
     bco2stock:= 1900000;

     minint:= 1000;
     WITH climate DO BEGIN
          biodiv:= 1;
          ch4add:= 0;
          n2oadd:= 0;
          sf6add:= 0;
          so2add:= 0;
          rfold:= 0;
          temp:= 0.20;
          dt:= 0.004986;
          sea:= 0;
          ds:= 0.001255;
          cbox[1]:= 283.53;
          cbox[2]:= 5.62;
          cbox[3]:= 6.29;
          cbox[4]:= 2.19;
          cbox[5]:= 0.15;
          acco2^[0]:= 0;
          acch4^[0]:= 1222;
          acn2o^[0]:= 296;
          acsf6^[0]:= sf6pre;
          FOR i:= 1 TO 5 DO
              acco2^[0]:= acco2^[0] + cbox[i];
          ch4n2o:= interact(ch4pre,n2opre)
     END;
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO BEGIN
              decade:= 0;
              kant:= 0;
              income:= GDP0;
              so2:= so20;
              population:= pop0;
              popdens:= population/area*1000000;
              IF riskave = 1 THEN
                 welfare:= log(1000*(1-consleak)*GDP0/pop0)
              ELSE
                  welfare:= pow(1000*(1-consleak)*GDP0/pop0,1-riskave)/(1-riskave);
              YpC:= 1000*GDP0/pop0;                     (* ????????? *)
              urbpop:= (0.031*root(ypc)-0.011*root(popdens))/(1+0.031*root(ypc)-0.011*root(popdens));
              energint:= 1;                             (* confirmed *)
              energuse:= GDP0;
              ryg:= 0;
              ch4red:= 0;
              n2ored:= 0;
              ch4cost:= 0;
              n2ocost:= 0;
              reei:= 0;                                 (* confirmed *)
              rcei:= 0;                                 (* confirmed *)
              seei:= 0;
              scei:= 0;
              htax:= 0;
              know:= 1;
              emissint:= emissint0;
              emission:= emissint*energuse;
              emiss0:= emission;                        (* confirmed *)
              IF emiss0/GDP0 < minint THEN
                 minint:= emiss0/GDP0;
              cdr:= 1;

              agrate:= agrbm*pow(0.005/dbst,agnl)*agtime;
              wetmeas:= 0;
              wettot:= 0;
              dryland:= 0;
              drycost:= 0;
              wetcost:= 0;
              leave:= 0;
              cumleave:= 0;
              cumenter:= 0;
              entercost:= 0;
              leavecost:= 0;
              watech:= 1;
              cumaeei:= 1;
              regtmp:= bregtmp*climate.temp;
              regdt:= bregtmp*climate.dt;
              eloss:= 0;
              sloss:= 0
     END;
END;

(* TAX PROCEDURES *)

PROCEDURE SetTax2;
(* sets all taxes to zero *)

VAR i, j: word;

BEGIN
     FOR i:= 1 TO NoReg DO
         FOR j:= 1 TO NoPer DO
             economy[i].tax[j]:= 0;
END;

(* METHANE PROCEDURES *)

PROCEDURE SetCH42;
(* sets all methane reduction to zero *)

VAR i: word;

BEGIN
     FOR i:= 1 TO NoReg DO
         economy[i].ch4red:= 0;
END;

(* NITROUS OXIDE PROCEDURES *)

PROCEDURE SetN2O2;
(* sets all nitrous oxide reduction to zero *)

VAR i: word;

BEGIN
     FOR i:= 1 TO NoReg DO
         economy[i].n2ored:= 0;
END;

(* FORESTRY PROCEDURES *)

PROCEDURE SetInflFor(VAR country: regrec; t: word);
(* determines the influence of COUNTRY.FORESTRY on the economy of COUNTRY *)

VAR fca, fcd, fa, fd, th1, th2: double;

BEGIN
     WITH country DO BEGIN
          IF forestry[optper] > 0 THEN BEGIN
             th1:= forconst - forlinear + root(quad(forconst-forlinear)-4*forconst*forsquare);
             th1:= th1/2/forsquare;
             th2:= (3*forconst/2 - forlinear)/2/forsquare;
             IF forestry[optper] < (th1 + th2) THEN BEGIN
                fd:= forestry[optper];
                fa:= 0
             END
             ELSE IF forestry[optper] < (2*th1 + th2) THEN BEGIN
                  fd:= th1 + th2;
                  fa:= forestry[optper] - fd
             END
             ELSE BEGIN
                  fa:= th1;
                  fd:= forestry[optper] - fa
             END;
             IF fd > forestemm^[t] THEN BEGIN    (* !!! *)
                fd:= forestemm^[t];
                fa:= forestry[optper] - fd
             END;
             IF fd > 0 THEN BEGIN
                fcd:= 2*(forconst + forlinear*fd + forsquare*quad(fd))/3;
                IF fcd < forconst*fd THEN
                   fcd:= 2*forconst*fd/3
             END
             ELSE
                 fcd:= 0;
             IF fa > 0 THEN BEGIN
                fca:= forconst + forlinear*fa + forsquare*quad(fa);
                IF fca < forconst*fa THEN
                   fca:= forconst*fa
             END
             ELSE
                 fca:= 0;
             forcost:= fca + fcd
          END
          ELSE
              forcost:= 0
     END
END;

PROCEDURE SetFor2;
(* sets all forestry to zero *)

VAR i, j: word;

BEGIN
     FOR i:= 1 TO NoReg DO
         FOR j:= 1 TO NoPer DO
             economy[i].forestry[j]:= 0;
END;

(* SETTINGS *)

PROCEDURE SetMeasures1;
(* sets all measures to zero *)

BEGIN
     SetTax2;
     SetFor2;
     SetN2O2;
     SetCH42
END;

(* CALCULATING PROCEDURES *)

PROCEDURE Calculate(VAR country: regrec; t: word);

VAR i, j, decade: word;
    s, h, henter, excessloss, popold, ic, dc, x, noemer, teller, seef, time: double;
    ygrowth: double;
    incth: boolean;

BEGIN
     time:= t;
     WITH country, climate DO BEGIN
          IF t > 55 THEN BEGIN
             ygrowth:= (1+0.01*(pgrowth^[t-1]+pgadd*(t-50)))*(1+0.01*(ypcgrowth^[t-1]+ecgradd*(t-50))) - 1;
             population:= (1+0.01*(pgrowth^[t-1]+pgadd*(t-50)))*population;
             energint:= (1 - 0.01*(aeei^[t-1] + aeeiadd*(t-50)) - reei)*energint;
             emissint:= (1 - 0.01*(acei^[t-1] + aceiadd*(t-50)) - rcei)*emissint;
             watech:= watech*(1-watechrate);
             SO2:= SO2*pow(1+0.01*(pgrowth^[t-1]+pgadd*(t-50)),so2pop)
                   *pow(1+0.01*(ypcgrowth^[t-1]+ecgradd*(t-50)),so2inc)
                   *pow(1-0.01*(aeei^[t]+aeeiadd*(t-50))-reei-0.01*(acei^[t]+aceiadd*(t-50))-rcei,so2carb);
          END
          ELSE BEGIN
               ygrowth:= (1+0.01*pgrowth^[t-1])*(1+0.01*ypcgrowth^[t-1]) - 1;
               population:= (1+0.01*pgrowth^[t-1])*population;
               energint:= (1 - 0.01*aeei^[t-1])*energint;
               emissint:= (1 - 0.01*acei^[t-1])*emissint;
               SO2:= SO2*pow(1+0.01*pgrowth^[t-1],so2pop)
                     *pow(1+0.01*ypcgrowth^[t-1],so2inc)
                     *pow(1-0.01*aeei^[t]-0.01*acei^[t],so2carb);
          END;
          IF t <= 40 THEN
             sf6:= (sf60 + sf6gdp*(income-GDP90) + sf6ypc*(ypc-ypc90))*(1+(time-40)/40)
          ELSE IF t <= 60 THEN
              sf6:= (sf60 + sf6gdp*(income-GDP90) + sf6ypc*(ypc-ypc90))*(1+(time-40)/40)
          ELSE
              sf6:= (sf60 + sf6gdp*(income-GDP90) + sf6ypc*(ypc-ypc90))*(1+(60-40)/40)*pow(0.99,time-60);
          IF sf6 < 0 THEN
             sf6:= 0;
          income:= (1 + ygrowth - taxmp*ryg - ch4cost - n2ocost)*income - formp*forcost*income/GDP0;
          IF (t > 50) AND (t <= 60) THEN
             income:= income + oda;
          popold:= population;
          ypcold:= ypc;
          IF population < 0.000000001 THEN
             YpC:= 0
          ELSE
              YpC:= 1000*income/population;
          IF t = 50 THEN
             ypc00:= ypc;
          agrish:= agrish0*pow(ypc/ypc90,-agel);
          energuse:= (1-seei)*energint*income;
          emission:= (1-scei)*emissint*energuse;
          IF income > 0 THEN BEGIN
             cummemm:= cummemm+emission;
             globcum:= globcum+emission
          END
          ELSE BEGIN
               globcum:= globcum-cummemm;
               cummemm:= 0
          END;
          IF t < startyear THEN
             MCh:= 0;
          IF t = startyear THEN
             MCh:= MC;
          IF t > startyear THEN BEGIN
             IF radforc > rfold THEN
                MCh:= MCh*1.05
             ELSE
                 MCh:= MCh/1.05;
(*             write(t+1950:4,' ',name,' ',MCh:10:4,' ',acco2^[t]:10:4);readln; *)
             taxpar:= 0.784 - 0.084*root(emission/income-minint);
             IF taxpar < 0.0784 THEN
                taxpar:= 0.0784;
             IF (name = 'USA') THEN BEGIN
                htax:= MCh*emission*know*globknow/2/taxpar/income/1000;
                ch4red:= 21*22/6/1000*MCh*ch4em^[t]/2/ch4par1/ch4par2/ch4par2/income/1000;
                n2ored:= 310*7/6/1000*MCh*n2oem^[t]/2/n2opar1/n2opar2/n2opar2/income/1000;
             END
             ELSE IF (name = 'CAN') THEN BEGIN
                htax:= MCh*emission*know*globknow/2/taxpar/income/1000;
                ch4red:= 21*22/6/1000*MCh*ch4em^[t]/2/ch4par1/ch4par2/ch4par2/income/1000;
                n2ored:= 310*7/6/1000*MCh*n2oem^[t]/2/n2opar1/n2opar2/n2opar2/income/1000;
             END
             ELSE IF (name = 'WEU') THEN BEGIN
                htax:= MCh*emission*know*globknow/2/taxpar/income/1000;
                ch4red:= 21*22/6/1000*MCh*ch4em^[t]/2/ch4par1/ch4par2/ch4par2/income/1000;
                n2ored:= 310*7/6/1000*MCh*n2oem^[t]/2/n2opar1/n2opar2/n2opar2/income/1000;
             END
             ELSE IF (name = 'JPK') THEN BEGIN
                htax:= MCh*emission*know*globknow/2/taxpar/income/1000;
                ch4red:= 21*22/6/1000*MCh*ch4em^[t]/2/ch4par1/ch4par2/ch4par2/income/1000;
                n2ored:= 310*7/6/1000*MCh*n2oem^[t]/2/n2opar1/n2opar2/n2opar2/income/1000;
             END
             ELSE IF (name = 'ANZ') THEN BEGIN
                htax:= MCh*emission*know*globknow/2/taxpar/income/1000;
                ch4red:= 21*22/6/1000*MCh*ch4em^[t]/2/ch4par1/ch4par2/ch4par2/income/1000;
                n2ored:= 310*7/6/1000*MCh*n2oem^[t]/2/n2opar1/n2opar2/n2opar2/income/1000;
             END
             ELSE IF (name = 'EEU') THEN BEGIN
                htax:= MCh*emission*know*globknow/2/taxpar/income/1000;
                ch4red:= 21*22/6/1000*MCh*ch4em^[t]/2/ch4par1/ch4par2/ch4par2/income/1000;
                n2ored:= 310*7/6/1000*MCh*n2oem^[t]/2/n2opar1/n2opar2/n2opar2/income/1000;
             END
             ELSE IF (name = 'FSU') AND (t >= startyear + delaybric) THEN BEGIN
                IF t < startyear + delaybric + transition THEN
                   htax:= (t-startyear-delaybric)/transition*MCh*emission*know*globknow/2/taxpar/income/1000
                ELSE
                    htax:= MCh*emission*know*globknow/2/taxpar/income/1000;
                ch4red:= 21*22/6/1000*MCh*ch4em^[t]/2/ch4par1/ch4par2/ch4par2/income/1000;
                n2ored:= 310*7/6/1000*MCh*n2oem^[t]/2/n2opar1/n2opar2/n2opar2/income/1000;
             END
             ELSE IF (name = 'SAM') AND (t >= startyear + delaybric) THEN BEGIN
                IF t < startyear + delaybric + transition THEN
                   htax:= (t-startyear-delaybric)/transition*MCh*emission*know*globknow/2/taxpar/income/1000
                ELSE
                    htax:= MCh*emission*know*globknow/2/taxpar/income/1000;
                ch4red:= 21*22/6/1000*MCh*ch4em^[t]/2/ch4par1/ch4par2/ch4par2/income/1000;
                n2ored:= 310*7/6/1000*MCh*n2oem^[t]/2/n2opar1/n2opar2/n2opar2/income/1000;
             END
             ELSE IF (name = 'SAS') AND (t >= startyear + delaybric) THEN BEGIN
                IF t < startyear + delaybric + transition THEN
                   htax:= (t-startyear-delaybric)/transition*MCh*emission*know*globknow/2/taxpar/income/1000
                ELSE
                    htax:= MCh*emission*know*globknow/2/taxpar/income/1000;
                ch4red:= 21*22/6/1000*MCh*ch4em^[t]/2/ch4par1/ch4par2/ch4par2/income/1000;
                n2ored:= 310*7/6/1000*MCh*n2oem^[t]/2/n2opar1/n2opar2/n2opar2/income/1000;
             END
             ELSE IF (name = 'CHI') AND (t >= startyear + delaybric) THEN BEGIN
                IF t < startyear + delaybric + transition THEN
                   htax:= (t-startyear-delaybric)/transition*MCh*emission*know*globknow/2/taxpar/income/1000
                ELSE
                    htax:= MCh*emission*know*globknow/2/taxpar/income/1000;
                ch4red:= 21*22/6/1000*MCh*ch4em^[t]/2/ch4par1/ch4par2/ch4par2/income/1000;
                n2ored:= 310*7/6/1000*MCh*n2oem^[t]/2/n2opar1/n2opar2/n2opar2/income/1000;
             END
             ELSE IF (name = 'MDE') AND (t >= startyear + delayrow) THEN BEGIN
                IF t < startyear + delayrow + transition THEN
                   htax:= (t-startyear-delayrow)/transition*MCh*emission*know*globknow/2/taxpar/income/1000
                ELSE
                    htax:= MCh*emission*know*globknow/2/taxpar/income/1000;
                ch4red:= 21*22/6/1000*MCh*ch4em^[t]/2/ch4par1/ch4par2/ch4par2/income/1000;
                n2ored:= 310*7/6/1000*MCh*n2oem^[t]/2/n2opar1/n2opar2/n2opar2/income/1000;
             END
             ELSE IF (name = 'CAM') AND (t >= startyear + delayrow) THEN BEGIN
                IF t < startyear + delayrow + transition THEN
                   htax:= (t-startyear-delayrow)/transition*MCh*emission*know*globknow/2/taxpar/income/1000
                ELSE
                    htax:= MCh*emission*know*globknow/2/taxpar/income/1000;
                ch4red:= 21*22/6/1000*MCh*ch4em^[t]/2/ch4par1/ch4par2/ch4par2/income/1000;
                n2ored:= 310*7/6/1000*MCh*n2oem^[t]/2/n2opar1/n2opar2/n2opar2/income/1000;
             END
             ELSE IF (name = 'SEA') AND (t >= startyear + delayrow) THEN BEGIN
                IF t < startyear + delayrow + transition THEN
                   htax:= (t-startyear-delayrow)/transition*MCh*emission*know*globknow/2/taxpar/income/1000
                ELSE
                    htax:= MCh*emission*know*globknow/2/taxpar/income/1000;
                ch4red:= 21*22/6/1000*MCh*ch4em^[t]/2/ch4par1/ch4par2/ch4par2/income/1000;
                n2ored:= 310*7/6/1000*MCh*n2oem^[t]/2/n2opar1/n2opar2/n2opar2/income/1000;
             END
             ELSE IF (name = 'NAF') AND (t >= startyear + delayrow) THEN BEGIN
                IF t < startyear + delayrow + transition THEN
                   htax:= (t-startyear-delayrow)/transition*MCh*emission*know*globknow/2/taxpar/income/1000
                ELSE
                    htax:= MCh*emission*know*globknow/2/taxpar/income/1000;
                ch4red:= 21*22/6/1000*MCh*ch4em^[t]/2/ch4par1/ch4par2/ch4par2/income/1000;
                n2ored:= 310*7/6/1000*MCh*n2oem^[t]/2/n2opar1/n2opar2/n2opar2/income/1000;
             END
             ELSE IF (name = 'SSA') AND (t >= startyear + delayrow) THEN BEGIN
                IF t < startyear + delayrow + transition THEN
                   htax:= (t-startyear-delayrow)/transition*MCh*emission*know*globknow/2/taxpar/income/1000
                ELSE
                    htax:= MCh*emission*know*globknow/2/taxpar/income/1000;
                ch4red:= 21*22/6/1000*MCh*ch4em^[t]/2/ch4par1/ch4par2/ch4par2/income/1000;
                n2ored:= 310*7/6/1000*MCh*n2oem^[t]/2/n2opar1/n2opar2/n2opar2/income/1000;
             END
             ELSE IF (name = 'SIS') AND (t >= startyear + delayrow) THEN BEGIN
                IF t < startyear + delayrow + transition THEN
                   htax:= (t-startyear-delayrow)/transition*MCh*emission*know*globknow/2/taxpar/income/1000
                ELSE
                    htax:= MCh*emission*know*globknow/2/taxpar/income/1000;
                ch4red:= 21*22/6/1000*MCh*ch4em^[t]/2/ch4par1/ch4par2/ch4par2/income/1000;
                n2ored:= 310*7/6/1000*MCh*n2oem^[t]/2/n2opar1/n2opar2/n2opar2/income/1000;
             END
             ELSE BEGIN
                 htax:= 0;
                 ch4red:= 0;
                 n2ored:= 0
(*                 htax:= (MCh/6)*emission*know*globknow/2/taxpar/income/1000;
                 ch4red:= 21*22/6/1000*(MCh/6)*ch4em^[t]/2/ch4par1/ch4par2/ch4par2/income/1000;
                 n2ored:= 310*7/6/1000*(MCh/6)*n2oem^[t]/2/n2opar1/n2opar2/n2opar2/income/1000;*)
             END;
             (*htax:= 2*htax;*)
             IF htax < 0 THEN
                htax:= 0;
             IF htax > 0.99 THEN
                htax:= 0.99;
             IF ch4red < 0 THEN
                ch4red:= 0;
             IF ch4red > 0.99 THEN
                ch4red:= 0.99;
             IF n2ored < 0 THEN
                n2ored:= 0;
             IF n2ored > 0.99 THEN
                n2ored:= 0.99;
             ryg:= taxpar*quad(htax)/know/globknow;
             perm:= 1-0.01*MCh/(1+0.01*MCh);
             reei:= perm*0.5*htax;
             IF reei > 0.99 THEN
                reei:= 0.99;
             IF MCh < 100 THEN
                rcei:= perm*0.5*100*quad(htax/100)
             ELSE
                 rcei:= perm*0.5*htax;
             IF rcei > 0.99 THEN
                rcei:= 0.99;
             ch4cost:= ch4par1*ch4par2*ch4par2*ch4red*ch4red;
             n2ocost:= n2opar1*n2opar2*n2opar2*n2ored*n2ored;
             seei:= 0.9*seei + (1-perm)*0.5*htax*1.7;
             IF seei > 0.99 THEN
                seei:= 0.99;
             IF htax < 100 THEN
                scei:= 0.9*scei + (1-perm)*0.5*100*quad(htax/100)*1.7
             ELSE
                 scei:= 0.9*scei + (1-perm)*0.5*htax*1.7;
             IF scei > 0.99 THEN
                scei:= 0.99;
             know:= know*root(1+knowpar*htax);
             IF know > 3.16 THEN
                know:= 3.16;
             globknow:= globknow*root(1+knowgpar*htax);
             IF globknow > 3.16 THEN
                globknow:= 3.16;
             setinflfor(country,t);
          END;

          species:= ((1-bioshare)*spbm+bioshare*biodiv*spbm)/valbase*
                    ypc/valinc/(1+ypc/valinc)/valadj*ypc*population/1000*
                    regdt/bregtmp/dbsta/(1+regdt/bregtmp/dbsta);
          water:= wrbm*watech*GDP90*pow(income/GDP90,wrel)*pow(regtmp/bregtmp,wrnl);
	  IF water > 0.1*income THEN
	     water:= 0.1*income;
          forests:= forbm*income*pow(ypc/ypc90,-forel)*(0.5*pow(regtmp/bregtmp,fornl)+0.5*ln(acco2^[t-1]/co2pre)*forco2);
          IF forests > 0.01*income THEN
             forests:= 0.01*income;
          heating:= hebm*cumaeei*(1-seei)*GDP90*pow(regtmp/bregtmp,henl)*pow(ypc/ypc90,heel)*population/pop90;
          IF heating > 0.05*income THEN
	     heating:= 0.05*income;
          cooling:= cebm*cumaeei*(1-seei)*GDP90*pow(regtmp/bregtmp,cenl)*pow(ypc/ypc90,ceel)*population/pop90;
          IF t > 50 THEN
             cumaeei:= cumaeei*(1-0.01*(aeei^[t]+aeeiadd*(t-50))-reei);
          dengue:= 0*dfbs*population*dfch*pow(regtmp/bregtmp,dfnl)*pow(ypc/ypc90,vbel);
          schisto:= 0*smbs*population*smch*pow(regtmp/bregtmp,smnl)*pow(ypc/ypc90,vbel);
          IF schisto < -smbs*population*pow(ypc/ypc90,vbel) THEN
             schisto:= -0*smbs*population*pow(ypc/ypc90,vbel);
          malaria:= 0*malbs*population*malch*pow(regtmp/bregtmp,malnl)*pow(ypc/ypc90,vbel);
          IF (temp90-0.49*bregtmp) > 0 THEN BEGIN
             diadead:= 0*diamort*population*pow(ypc/ypc90,diamortel)
                       *(pow((temp90-0.49*bregtmp+regtmp)/(temp90-0.49*bregtmp),diamortnl)-1);
             diasick:= 0*diayld*population*pow(ypc/ypc90,diayldel)
                       *(pow((temp90-0.49*bregtmp+regtmp)/(temp90-0.49*bregtmp),diayldnl)-1)
          END
          ELSE BEGIN
               diadead:= 0;
               diasick:= 0
          END;
          agrate:= agrbm*pow(regdt/bregtmp/dbst,agnl) + (1-1/agtime)*agrate;
          aglevel:= aglparl*regtmp/bregtmp + aglparq*pow(regtmp/bregtmp,2);
          agco2:= agcbm*ln(acco2^[t-1]/co2pre);
          agcost:= (agrate+aglevel+agco2)*agrish*income;
          IF agcost > agrish*income THEN
             agcost:= agrish*income;

          plus:= plus90*pow(ypc/ypc90,plusel);
	  IF plus > 1 THEN
	     plus:= 1;
          urbpop:= (0.031*root(ypc)-0.011*root(popdens))/(1+0.031*root(ypc)-0.011*root(popdens))
                   /(1+urbcorr/(1+0.001*pow(time-40,2)));
          basecardvasc:= cardvasc90 + cvlin*(plus-plus90);
	  IF basecardvasc > 1 THEN
	     basecardvasc:= 1;
          baseresp:= resp90 + rlin*(plus-plus90);
	  IF baseresp > 1 THEN
	     baseresp:= 1;

          cardheat:= (chplbm*plus + chmlbm*(1-plus))*regtmp/bregtmp +
                     (chpqbm*plus + chmqbm*(1-plus))*pow(regtmp/bregtmp,2);
          cardheat:= cardheat*urbpop*population*10;
          IF cardheat > 1000*maxcardvasc*basecardvasc*urbpop*population THEN
             cardheat:= 1000*maxcardvasc*basecardvasc*urbpop*population;
          IF cardheat < 0 THEN
             cardheat:= 0;

          resp:= rlbm*regtmp/bregtmp + rqbm*pow(regtmp/bregtmp,2);
          resp:= resp*urbpop*population*10;
          IF resp > 1000*maxcardvasc*baseresp*urbpop*population THEN
             resp:= 1000*maxcardvasc*baseresp*urbpop*population;
          IF resp < 0 THEN
             resp:= 0;

          cardcold:= (ccplbm*plus + ccmlbm*(1-plus))*regtmp/bregtmp +
                     (ccpqbm*plus + ccmqbm*(1-plus))*pow(regtmp/bregtmp,2);
          cardcold:= cardcold*population*10;
          IF cardcold < -1000*maxcardvasc*basecardvasc*population THEN
             cardcold:= -1000*maxcardvasc*basecardvasc*population;
          IF cardcold > 0 THEN
             cardcold:= 0;

          dead:= dengue + schisto + malaria + cardheat + cardcold + resp + diadead;

          yll:= d2ld*dengue+d2ls*schisto+d2lm*malaria+d2lc*cardheat+d2lc*cardcold+d2lr*resp;
          yld:= d2dd*dengue+d2ds*schisto+d2dm*malaria+d2dc*cardheat+d2dc*cardcold+d2dr*resp+diasick;
          deadcost:= vsl*ypc*dead/1000000000;
          (*deadcost:= vyll*ypc*yll/1000000000;*)
          morbcost:= vmorb*ypc*yld/1000000000;

          wetval:= wvbm/valbase*ypc/valinc/(1+ypc/valinc)/valadj*ypc;
          IF (wetmax > 0) THEN
             IF wettot > wetmax*(1-bioshare/(1+bioshare)) THEN
                wetval:= wetval*2
             ELSE
                 wetval:= wetval*((1-bioshare)+bioshare*wetmax/(wetmax-wettot));
          dryval:= dvbm*popdens*ypc/1000/incdens;
          npprotcost:= pc*ds*100*(1+discrate+ygrowth)/(discrate+ygrowth);
          IF npprotcost < 0 THEN
             npprotcost:= 0;
          npwetcost:= wetval*wmbm*ds/(discrate+ygrowth);
          IF npwetcost < 0 THEN
             npwetcost:= 0;
          npdrycost:= dryval*dlbm*ds*(1+ygrowth)*(discrate+ygrowth)/discrate/discrate;
          IF npdrycost < 0.00000001 THEN BEGIN
             npdrycost:= 0;
             protlev:= 0
          END
          ELSE
              protlev:= 1-0.5*(npprotcost+npwetcost)/npdrycost;
          IF protlev < 0 THEN
             protlev:= 0;
          wetland:= wlbm*sea;
          IF ds > 0 THEN
             wetmeas:= wmbm*ds*protlev + wetmeas;
          wettot:= wetland + wetmeas;
          IF ds > 0 THEN
             wetann:= (wlbm + wmbm*protlev)*ds
          ELSE
              wetann:= 0;
          IF wettot > wetmax THEN BEGIN
             wettot:= wetmax;
             wetann:= 0
          END;
          wetcost:= wetval*wetann/1000 + wetdel*wetcost;
          IF ds > 0 THEN BEGIN
             dryland:= dlbm*ds*(1-protlev) + dryland;
             area:= area - dlbm*ds*(1-protlev);
             drycost:= dryval*dlbm*ds*(1-protlev)/1000 + drydel*drycost;
          END;
	  IF drycost < 0 THEN
	     drycost:= 0;
          protcost:= protlev*pc*ds/10;
	  IF protcost < 0 THEN
	     protcost:= 0;
          enter:= 0;
          FOR j:= 1 TO NoReg DO
              enter:= enter + immigrate[j]*economy[j].leave;
          leave:= coastpd*popdens*dlbm*ds*(1-protlev);
          IF leave < 0 THEN
             leave:= 0;
          cumleave:= leave + cumleave;
          cumenter:= enter + cumenter;
          leavecost:= emcst*ypc*leave/1000000000 + leavedel*leavecost;
          entercost:= immcst*ypc*enter/1000000000 + (1-1/entertime)*entercost;

          eloss:= 1*(- 1*water - 1*forests - 1*heating - 1*cooling - 1*agcost + 1*drycost + 1*protcost + 1*entercost);
          IF eloss > income THEN
             eloss:= income;
          sloss:= 1*(species + deadcost + morbcost + wetcost + leavecost);
          IF t >= 40 THEN
             population:= population + (enter - leave)/1000000;
          IF dead > 0 THEN
             population:= population - dead/1000000;
	  IF population <= 0.000001 THEN
	     population:= 0.000001;
          popdens:= population/area*1000000;
          welfare:= (1-consleak)*income - eloss;
          IF t > 50 THEN
             income:= income - consleak*eloss/10;
          IF (population < 0.00000001) OR ((sloss+kant) >= welfare) OR (welfare <= 0) THEN
             welfare:= -100000000
          ELSE BEGIN
              welfare:= 1000*(welfare-sloss-kant)/population;
              IF riskave = 1 THEN
                 welfare:= log(welfare)
              ELSE
                  welfare:= pow(welfare,1-riskave)/(1-riskave)
          END;
          IF income < 0.01*population THEN
             income:= 0.01*population;
          eloss:= eloss/income;
          sloss:= sloss/income;
          IF ((optper < 11) AND (t = 40+10*optper)) OR ((optper > 10) AND (t = 25*optper-125)) THEN BEGIN
             cdr:= 1;
             hNPVW:= welfare
          END;
          IF ((optper < 11) AND (t > 40+10*optper)) OR ((optper > 10) AND (t > 25*optper-125))THEN BEGIN
             cdr:= (1+discrate)*cdr;
             IF hNPVW >= 1.0E-309 THEN
                hNPVW:= hNPVW + welfare/cdr
          END
     END
END;

PROCEDURE Write1(VAR country: regrec; t: word; VAR els,sls,
                     ent,mal,sch,den,car,res,lea,inc,wel,sul,pop,mc: text);

BEGIN
     WITH country, climate DO BEGIN
          write(ent,enter:10:4,' ');
          write(lea,leave:10:4,' ');
          write(mal,malaria:10:4,' ');
          write(sch,schisto:10:4,' ');
          write(den,diadead:10:4,' ');
          write(car,cardheat+cardcold:10:4,' ');
          write(res,resp:10:4,' ');
          IF income > 0.000001 THEN BEGIN
             write(els,eloss*100:8:4,' ');
             write(sls,sloss*100:8:4,' ');
             IF t >= 40 THEN BEGIN
                write(mc,2*taxpar*tax[trunc((t-40)/10)]*income*1000/emission/know/globknow:8:2,' ');
write(mc,-taxpar*tax[trunc((t-40)/10)]*income*1000/know/globknow/emission/root(1+tax[trunc((t-40)/10)]):8:2,' ')
             END
             ELSE
                 write(mc,0,' ',0,' ');
          END
          ELSE BEGIN
               write(els,0,' ');
               write(sls,0,' ');
               write(mc,0,' ')
          END;
          write(inc,income:9:3,' ');
          write(wel,emission:6:0,' ');
          write(sul,so2:8:4);
          write(pop,population:9:3,' ');
     END
END;

PROCEDURE Write2(VAR country: regrec; t: word; VAR ch4out, n2oout, sf6out: text);

BEGIN
     WITH country, climate DO BEGIN
          write(ch4out,ch4em^[t]*(1-ch4red):8:4,' ');
          write(n2oout,n2oem^[t]*(1-n2ored):8:4,' ');
          write(sf6out,sf6:8:4,' ');
     END
END;

PROCEDURE WriteM(VAR country: regrec; t: word; VAR els,sls,inc,pop: text);

BEGIN
     WITH country, climate DO BEGIN
          IF income > 0.000001 THEN BEGIN
             write(els,eloss*100:14:10,' ');
             write(sls,sloss*100:14:10,' ');
          END
          ELSE BEGIN
               write(els,0,' ');
               write(sls,0,' ');
          END;
          write(inc,income:12:6,' ');
          write(pop,population:12:6,' ');
     END
END;

PROCEDURE CalcClimate(j: word);
(* calculates the climatic variables at time J *)

VAR mco2, emfforest: double;
    i, dec: word;
    rfCO2, rfCH4, rfN2O, rfSF6, rfSO2, radforcemf: double;
    globch4, globn2o, globsf6, addco2, addch4, addn2o, addsf6, co2sens: double;

BEGIN
     dec:= economy[1].decade;
     addco2:= 0;
     addch4:= 0;
     addn2o:= 0;
     addsf6:= 0;
     WITH climate DO BEGIN
          mco2:= 0;
          FOR i:= 1 TO NoReg DO
              WITH economy[i] DO BEGIN
                   mco2:= mco2 + emission + forestemm^[j]
              END;
          IF j > 50 THEN BEGIN
             biodiv:= nospecbase/nospecies;
             co2sens:= 0; (* 0 / 600 / 2600 / 7500 *)
             mco2:= mco2 + (temp-0.963703)*co2sens*bco2stock/1900000;
             bco2stock:= bco2stock - (temp-0.963703)*co2sens*bco2stock/1900000;
             IF bco2stock < 0 THEN
                bco2stock:= 0;
             FOR i:= 1 TO NoReg DO
                 WITH economy[i] DO
                      mco2:= mco2 + foremadd*(j-50) - forestry[dec]
          END;
          IF (j >= 10*mdp) AND (j < 10*(mdp+1)) THEN BEGIN
             IF MargCO2 THEN
                addco2:= 1
             ELSE IF MargCH4 THEN
                  addch4:= 1
             ELSE IF MargN2O THEN
                  addn2o:= 1
             ELSE IF MargSF6 THEN
                  addsf6:= 1
          END
          ELSE BEGIN
              addco2:= 0;
              addch4:= 0;
              addn2o:= 0;
              addsf6:= 0
          END;
(*          IF (j > 50) THEN
             mco2:= mco2*pow(0.994,j-50);
(* 988 whenflex, 998 annual, 997 tech, 984 tech+WF, 994 tech+ann *)
          FOR i:= 1 TO 5 DO
              cbox[i]:= cbox[i]*decay[i] + 0.000471*frac[i]*(mco2 + addmp*addco2);
          acco2^[j]:= cbox[1];
          FOR i:= 2 TO 5 DO
              acco2^[j]:= acco2^[j] + cbox[i];
          globch4:= 0;
          globn2o:= 0;
          globsf6:= 0;
          globso2:= 34.4;
          FOR i:= 1 TO NoReg DO BEGIN
              IF j > 50 THEN BEGIN
                 globch4:= globch4 + economy[i].ch4em^[j]*(1-economy[i].ch4red);
                 globn2o:= globn2o + economy[i].n2oem^[j]*(1-economy[i].n2ored);
                 globso2:= globso2 + economy[i].so2*(1-economy[i].seei-economy[i].scei);
                 globsf6:= globsf6 + economy[i].sf6
              END
              ELSE BEGIN
                   globch4:= globch4 + economy[i].ch4em^[j];
                   globn2o:= globn2o + economy[i].n2oem^[j];
                   globso2:= globso2 + economy[i].so2;
                   globsf6:= globsf6 + economy[i].sf6
              END
          END;
          IF j > 50 THEN BEGIN
             acch4^[j]:= acch4^[j-1] + 0.3597*(globch4 + ch4add*(j-50) + addmp*addch4)
                         - ch4decay*(acch4^[j-1]-ch4pre);
             acn2o^[j]:= acn2o^[j-1] + 0.2079*(globn2o + n2oadd*(j-50) + addmp*addn2o)
                         - n2odecay*(acn2o^[j-1]-n2opre);
             acsf6^[j]:= sf6pre + (acsf6^[j-1]-sf6pre)*(1-sf6decay)
                         + (globsf6 + sf6add*(j-50) + addmp*addsf6)/25.1;
             globso2:= globso2 + so2add*(j-50)
          END
          ELSE BEGIN
               acch4^[j]:= acch4^[j-1] + 0.3597*(globch4 + addmp*addch4)
                           - ch4decay*(acch4^[j-1]-ch4pre);
               acn2o^[j]:= acn2o^[j-1] + 0.2079*(globn2o + addmp*addn2o)
                         - n2odecay*(acn2o^[j-1]-n2opre);
               acsf6^[j]:= sf6pre + (acsf6^[j-1]-sf6pre)*(1-sf6decay)
                           + (globsf6 + addmp*addsf6)/25.1
          END;
          IF acch4^[j] < 0 THEN
             acch4^[j]:= 0;
          IF acn2o^[j] < 0 THEN
             acn2o^[j]:= 0;
          IF acsf6^[j] < 0 THEN
             acsf6^[j]:= 0;
          rfCO2:= 5.68*log(acco2^[j]/co2pre);
          rfCH4:= 0.031*(sqrt(acCH4^[j])-sqrt(ch4pre))
                       - interact(acch4^[j],n2opre) + CH4N2O;
          rfN2O:= 0.11*(sqrt(acN2O^[j])-sqrt(N2Opre))
                       - interact(ch4pre,acn2o^[j]) + CH4N2O;
          rfSF6:= 0.00052*(acsf6^[j]-sf6pre);
          rfSO2:= so2dir*globSO2/14.6 + so2ind*ln(1+globSO2/34.4)/ln(1+14.6/34.4) - 0.9;
          rfold:= radforc;
          radforc:= rfCO2 + rfCH4 + rfN2O + rfSF6 - rfSO2;
          radforcemf:= rfCO2 + rfCH4 + rfN2O + rfSF6;
(*write(j+1950:4,' ',rfCO2:8:4,' ',rfCH4:8:4,' ',rfN2O:8:4,' ',rfSF6:8:4,' ',rfSO2:8:4,' ',radforc:8:4);readln;*)
(*write(j+1950:4,' ',acsf6^[j]:10:4,' ',globsf6:10:4,' ',addmp:3:1,' ',addsf6:3:1,' ',rfSF6:16:14,' ',radforc:12:8);readln;*)

          IF (radforcemf > maxRF) AND (j <= 150) THEN BEGIN
             maxRF:= radforcemf;
             maxPCO2:= cbox[1];
             maxYR:= j+1950
          END;
          IF j=150 THEN BEGIN
             RF2100:= radforcemf;
             PCO2100:= cbox[1]
          END;
          IF j=350 THEN
             RF2300:= radforcemf;
          dt:= delaytemp*temps*radforc - delaytemp*temp;
          temp:= dt + temp;
          IF temp > geoth THEN BEGIN
             dt:= 0;
             temp:= geoth
          END;
          IF j = 150 THEN
             T2100:= temp;
          FOR i:= 1 TO NoReg DO BEGIN
              IF scengmt THEN BEGIN
                 economy[i].regtmp:= temp*economy[i].bregtmp + economy[i].scentemp^[j];
                 economy[i].regdt:= dt*economy[i].bregtmp + economy[i].scentemp^[j] - economy[i].scentemp^[j-1];
                 IF economy[i].regdt < 0 THEN
                    economy[i].regdt:= -economy[i].regdt
              END
              ELSE BEGIN
                   economy[i].regtmp:= temp*economy[i].bregtmp;
                   economy[i].regdt:= dt*economy[i].bregtmp;
                   IF economy[i].regdt < 0 THEN
                      economy[i].regdt:= -economy[i].regdt
              END
          END;
          IF j > 50 THEN
             nospecies:= nospecies*(1-bioloss-biosens*dt*dt/dbsta/dbsta);
          IF nospecies < nospecbase/100 THEN
             nospecies:= nospecbase/100;
(*          IF j < 150 THEN *)
             ds:= delaysea*seas*temp - delaysea*sea
(*          ELSE
              ds:= delaysea*(seas*temp + (j-150)*0.025) - delaysea*sea + 0.025*);
          sea:= ds + sea;
          IF j = 40+10*optper THEN
             toptper:= temp
     END
END;

PROCEDURE DrawRandom;

VAR i, j: word;
    hulp, fake,immsum: double;

BEGIN
    WITH climate DO BEGIN
         GenNormal(0,1,ch4add,n2oadd);
         GenNormal(0,1,sf6add,so2add);
         ch4add:= ch4add*ch4sd;
         n2oadd:= n2oadd*n2osd;
         sf6add:= sf6add*sf6sd;
         so2add:= so2add*so2sd;
(*GenNormal(1.491,0.334,temps,fake);  (*Ada's CS1 *)
         GenGamma(8.1270,0.3508,temps); (*mode 2.5; var 1*) (*Ada's CS2*)
(*         Gen3Perc(0.85,4.5,9.35,temps);*)
(*         GenGamma(2.52,3,temps); (* Ada's CS3 *)
(*         GenGamma(4.9218,0.6375,temps); (*mode 2.5; var 2*)
(*         GenGamma(4.27,1.16,temps);       (*mode 3.8; sd 2.4*)
         temps:= temps/6.3/ln(2);
(*         GenGamma(6.3584,0.0317,seas); (* mode 0.17; var 0.08^2 *)
(*         GenGamma(4.0083,0.0565,seas); (* mode 0.17; var 2*0.08^2 *)
(*         GenGamma(7.4729,0.0263,seas);      (* mode 0.17; sd 0.07 *)
         GenGamma(5.9959077,0.0612581,seas); (* mode 0.30604; var 0.15^2 *)
         GenTriang(8,16,1/ch4dec,hulp);
         ch4decay:= 1/hulp;
         GenTriang(100,170,1/n2odec,hulp);
         n2odecay:= 1/hulp;
         GenTriang(1600,4800,1/sf6dec,hulp);
         sf6decay:= 1/hulp;
         GenTriang(0.5*lifetemp,2*lifetemp,lifetemp,hulp);
         delaytemp:= 1/hulp;
         GenTriang(0.5*lifesea,2*lifesea,lifesea,hulp);
         delaysea:= 1/hulp;
         GenNormGTZ(lifeco2,0.25*lifeco2,hulp);
         decay[2]:= exp(-1/hulp);
         GenNormGTZ(lifeco3,0.25*lifeco3,hulp);
         decay[3]:= exp(-1/hulp);
         GenNormGTZ(lifeco4,0.25*lifeco4,hulp);
         decay[4]:= exp(-1/hulp);
         GenNormGTZ(lifeco5,0.25*lifeco5,hulp);
         decay[5]:= exp(-1/hulp);
    	 GenNormGTZ(biolossbg,biolosssd,bioloss);
    	 GenNormGTZ(biosensbg,biosenssd,biosens);
    	 GenTriang(0,1,biosharebg, bioshare);
    END;
    GenNormGTZ(watechratebg,watechratesd,watechrate);

    FOR i:= 1 TO NoReg DO
        WITH economy[i] DO BEGIN
             gennormal(0,1,pgadd,ecgradd);
             pgadd:= pgadd*pgsd;
             ecgradd:= ecgradd*ecgrsd;
             gennormal(0,1,aeeiadd,aceiadd);
             aeeiadd:= aeeiadd*aeeisd;
             aceiadd:= aceiadd*aceisd;

             gennormal(0,1,foremadd,fake);
             foremadd:= foremsd*foremadd;

             gennormGTZ(spbmbg,spbmsd,spbm);
             gennormGTZ(vslbg,vslsd,vsl);

             GenNormGTZ(vmorbbg,vmorbsd,vmorb);
             gennormGTZ(vyllbg,vyllsd,vyll);

             gennormGTZ(valincbg,valincsd,valinc);
             valadj:= valbase/valinc/(1+valbase/valinc);

             gennormGTZ(malchbg,malchsd,malch);
             gennormGTZ(malnlbg,malnlsd,malnl);
             gennormGTZ(dfchbg,dfchsd,dfch);
             gennormGTZ(dfnlbg,dfnlsd,dfnl);
             gennormGTZ(-smchbg,smchsd,smch);
             smch:= -smch;
             gennormGTZ(smnlbg,smnlsd,smnl);
             gennormGTZ(-vbelbg,vbelsd,vbel);
             vbel:= -vbel;

             gennormGTZ(forco2bg,forco2sd,forco2);

             GenNormGTZ(-diamortelbg,diamortelsd,diamortel);
             diamortel:= -diamortel;
             gennormGTZ(diamortnlbg,diamortnlsd,diamortnl);
             GenNormGTZ(-diayldelbg,diayldelsd,diayldel);
             diayldel:= -diayldel;
             gennormGTZ(diayldnlbg,diayldnlsd,diayldnl);

             gennormal(0,1,chplbm,chmlbm);
             gennormal(0,1,chpqbm,chmqbm);
             gennormal(0,1,ccplbm,ccmlbm);
             gennormal(0,1,ccpqbm,ccmqbm);
             gennormal(0,1,rlbm,rqbm);
             chplbm:= ccplbmbg + chplbmsd*chplbm;
             chmlbm:= chmlbmbg + chmlbmsd*chmlbm;
             chpqbm:= chpqbmbg + chpqbmsd*chpqbm;
             chmqbm:= chmqbmbg + chmqbmsd*chmqbm;
             ccplbm:= ccplbmbg + ccplbmsd*ccplbm;
             ccmlbm:= ccmlbmbg + ccmlbmsd*ccmlbm;
             ccpqbm:= ccpqbmbg + ccpqbmsd*ccpqbm;
             ccmqbm:= ccmqbmbg + ccmqbmsd*ccpqbm;
             rlbm:= rlbmbg + rlbmsd*rlbm;
             rqbm:= rqbmbg + rqbmsd*rqbm;
             gennormGTZ(cvlinbg,cvlinsd,cvlin);
             gennormGTZ(rlinbg,rlinsd,rlin);
             gennormGTZ(pluselbg,pluselsd,plusel);
             gennormGTZ(maxcardvascbg,maxcardvascsd,maxcardvasc);

             gennormal(0,1,wrbm,forbm);
             gennormGTZ(wrelbg,wrelsd,wrel);
             gennormGTZ(wrnlbg,wrnlsd,wrnl);
             gennormGTZ(forelbg,forelsd,forel);
             gennormGTZ(fornlbg,fornlsd,fornl);
             gennormGTZ(hebmbg,hebmsd,hebm);
	     gennormGTZ(-cebmbg,cebmsd,cebm);
             cebm:= -cebm;
             gennormGTZ(heelbg,heelsd,heel);
             gennormGTZ(henlbg,henlsd,henl);
             gennormGTZ(ceelbg,ceelsd,ceel);
             gennormGTZ(cenlbg,cenlsd,cenl);
             wrbm:= wrbmbg + wrbmsd*wrbm;
             forbm:= forbmbg + forbmsd*forbm;

             gennormGTZ(-agrbmbg,agrbmsd,agrbm);
             agrbm:= -agrbm;
             gennormGTZ(agtimebg-1,agtimesd,agtime);
             agtime:= agtime+1;
             gennormGTZ(agnlbg,agnlsd,agnl);
             gennormGTZ(agelbg,agelsd,agel);
             gennormal(0,1,aglbm,agopt);
             gennormGTZ(agcbmbg/ln(2),agcbmsd/ln(2),agcbm);
             aglbm:= aglbmbg + aglbmsd*aglbm;
             agopt:= agoptbg + agoptsd*agopt;
             aglparl:= -2*agopt*aglbm/(10.24-6.4*agopt);
             aglparq:= aglbm/(10.24-6.4*agopt);
             IF aglparq > 0 THEN BEGIN
                aglparq:= 0;
                aglparl:= aglbm/3.2
             END;

             gennormGTZ(wvbmbg,wvbmsd,wvbm);
             gennormGTZ(dvbmbg,dvbmsd,dvbm);
             gennormGTZ(wlbmbg,wlbmsd,wlbm);
             gennormGTZ(wmbmbg,wmbmsd,wmbm);
             gennormGTZ(pcbg,pcsd,pc);
             gennormGTZ(dlbmbg,dlbmsd,dlbm);

             genexponential(wetdelsd,wetdel);
             WHILE wetdel > 1 DO
                   genexponential(wetdelsd,wetdel);
             genexponential(drydelsd,drydel);
             WHILE drydel > 1 DO
                   genexponential(drydelsd,drydel);
             genexponential(leavedelsd,leavedel);
             WHILE leavedel > 1 DO
                   genexponential(leavedelsd,leavedel);

             FOR j:= 1 TO NoReg DO
                 genexponential(immigratesd[j],immigrate[j]);
             gennormGTZ(immigratebg[i],immigratesd[i],immigrate[i]);
             gennormGTZ(entertimebg-1,entertimesd,entertime);
             entertime:= entertime+1;
             immsum:= 0;
             FOR j:= 1 TO NoReg DO
                 immsum:= immsum + immigrate[j];
             FOR j:= 1 TO NoReg DO
                 immigrate[j]:= immigrate[j]/immsum;

             gennormGTZ(emcstbg,emcstsd,emcst);
             gennormGTZ(immcstbg,immcstsd,immcst);

             gengamma(5.8284,0.2071,taxmp);
             gengamma(5.8284,0.2071,formp);
(*             gengamma(5.8284,0.2071,cregmp);  (* var = 0.25, mode = 1 *)
(*             gengamma(9.8990,0.1124,taxmp);
             gengamma(9.8990,0.1124,formp);
             gengamma(9.8990,0.1124,cregmp);   (* var = 0.125, mode = 1 *)
        END;
    hulp:= 4.62;
    GenExponential(economy[7].trade[1], hulp);
    hulp:= 2.63;
    GenExponential(economy[8].trade[1], hulp);
    hulp:= 0.53;
    GenExponential(economy[9].trade[1], hulp);
    GenExponential(economy[10].trade[1], hulp);
    GenExponential(economy[13].trade[1], hulp);
    GenExponential(economy[16].trade[1], hulp);
    hulp:= 0.46;
    GenExponential(economy[11].trade[1], hulp);
    GenExponential(economy[12].trade[1], hulp);
    hulp:= 1.08;
    GenExponential(economy[14].trade[1], hulp);
    GenExponential(economy[15].trade[1], hulp);
    FOR i:= 7 TO NoReg DO BEGIN
        economy[i].trade[2]:= economy[i].trade[1]*4/5;
        economy[i].trade[3]:= economy[i].trade[1]*3/5;
        economy[i].trade[4]:= economy[i].trade[1]*2/5;
        economy[i].trade[5]:= economy[i].trade[1]*1/5
    END
END;

PROCEDURE RunEconomy;
(* calculates the economic and climatic variables for the entire period *)

VAR i, j, run : word;
    worldemm, oecdemm, ann1emm, maxkant, loss, totloss, transfer, source: double;

BEGIN
(*write('.');*)
     IF uncertain THEN BEGIN
        randseed:= 34514325;
        FOR run:= 1 TO runs DO BEGIN
            write(run:5);
            DrawRandom;
            ReInit;
            FOR i:= 1 TO NYear DO BEGIN
                IF i < 40 THEN
                   minint:= 1000;
                FOR j:= 1 TO NoReg DO BEGIN
                    Calculate(economy[j],i);
                    IF economy[j].emission/economy[j].income < minint THEN
                       minint:= economy[j].emission/economy[j].income
                END;
                IF ((i < 150) AND (i = 49+10*optper)) OR (i=149+25*optper) THEN
                   FOR j:= 1 TO NoReg DO BEGIN
                       emmrec[j]:= emmrec[j]*(run-1)/run + economy[j].emission/run;
                       IF economy[j].emission > ncemmres[j,optper] THEN
                          count[j]:= count[j]+1
                   END;
                CalcClimate(i)
            END;
            FOR i:= 1 TO NoReg DO
                economy[i].NPVW:= economy[i].hNPVW/run +
                                     (run-1)*economy[i].NPVW/run
        END
      END
     ELSE BEGIN
          InitImpactPar;
          ReInit;
          FOR i:= 1 TO NYear DO BEGIN
              maxkant:= 0;
              totloss:= 0;
              IF i < 40 THEN
                 minint:= 1000;
              FOR j:= 1 TO NoReg DO BEGIN
                  Calculate(economy[j],i);
                  loss:= (economy[j].eloss+economy[j].sloss);
                  totloss:= totloss + loss*economy[j].income;
                  IF loss > maxkant THEN
                     maxkant:= loss;
                  IF economy[j].emission/economy[j].income < minint THEN
                     minint:= economy[j].emission/economy[j].income
              END;
              IF kantian THEN
                 FOR j:= 1 TO NoReg DO
                     WITH economy[j] DO BEGIN
                          kant:= maxkant - (eloss+sloss);
                          kant:= kant*income
                       END;
              IF compens THEN
                 FOR j:= 1 TO NoReg DO
                     WITH economy[j] DO BEGIN
                          income:= income*(1+eloss+sloss) - totloss*cummemm/globcum;
                          IF income < 0.0000001 THEN
                             income:= 0
                     END;
              IF aid AND (i>50) THEN BEGIN
                 transfer:= 0;
                 source:= 0;
                 FOR j:= 1 TO NoReg DO
                     WITH economy[j] DO
                          IF ypcold > ypc THEN
                             transfer:= transfer + (ypcold-ypc)*population/1000
                          ELSE
                              source:= source + 0.01*income;
                 IF (transfer > 0) AND (source > 0) AND (transfer < source) THEN
                    FOR j:= 1 TO NoReg DO
                        WITH economy[j] DO
                             IF ypcold > ypc THEN
                                income:= ypcold*population/1000
                             ELSE
                                 income:= income*(1 - transfer/source);
              END;
              IF ((i < 150) AND (i = 49+10*optper)) OR (i=149+25*optper) THEN BEGIN
                 worldemm:= 0;
                 oecdemm:= 0;
                 FOR j:= 1 TO NoReg DO BEGIN
                     IF j < 4 THEN
                        oecdemm:= oecdemm + economy[j].emission;
                     worldemm:= worldemm + economy[j].emission (*+
                                economy[j].foremp*economy[j].forestemm^[i] -
                                economy[j].forestry[trunc(i/10)]*);
                     emmrec[j]:= economy[j].emission -
                                 gwpch4*economy[j].ch4em^[i]*economy[j].ch4red -
                                 gwpn2o*economy[j].n2oem^[i]*economy[j].n2ored
                 END;
                 ann1emm:= oecdemm + economy[4].emission (*- economy[2].emission*);
                 worldemm:= worldemm (*- economy[5].emission - economy[6].emission - economy[9].emission*);
                 globviol:= (worldemm > cemmres[optper])
              END;
              CalcClimate(i)
          END;
          C2200:= climate.acco2^[NYear];
          FOR i:= 1 TO NoReg DO
(*              IF C2200 > CMax THEN
                 economy[i].NPVW:= economy[i].hNPVW*
                                   (1-0.5*(noper-optper)/noper*(C2200-CMax)*(C2200-CMax)/(CBaU-CMax)/(CBaU-CMax))
              ELSE *)
                  economy[i].NPVW:= economy[i].hNPVW
     END
END;

PROCEDURE RunEconomy1;
(* calculates the economic and climatic variables for the entire period *)

VAR i, j, k, run : word;
    els,sls,eap,sap,ent,mal,sch,den,car,res,lea,inc,wel,pop,clim,mc,sul: text;
    ch4out, n2oout, sf6out: text;
    wunc,cunc,tunc,punc,gunc,iunc,eunc,runc: text;
    totpop,totinc,totimp,totemm,totred,transfer,source: double;
    loss, totloss, maxkant: double;

BEGIN
     writeln('enter runeconomy1');
     IF uncertain THEN BEGIN
        randseed:= 34514325;
        assign(wunc,sessionpathname+'welfunc.out');
        rewrite(wunc);
        assign(cunc,sessionpathname+'concunc.out');
        rewrite(cunc);
        assign(tunc,sessionpathname+'tempunc.out');
        rewrite(tunc);
        assign(punc,sessionpathname+'popunc.out');
        rewrite(punc);
        assign(gunc,sessionpathname+'incunc.out');
        rewrite(gunc);
        assign(iunc,sessionpathname+'impunc.out');
        rewrite(iunc);
        assign(eunc,sessionpathname+'emmunc.out');
        rewrite(eunc);
        assign(runc,sessionpathname+'redunc.out');
        rewrite(runc);
        FOR run:= 1 TO runs DO BEGIN
writeln(run:5);
            DrawRandom;
            ReInit;
            FOR i:= 1 TO NYear DO BEGIN
                IF i < 40 THEN
                   minint:= 1000;
                FOR j:= 1 TO NoReg DO BEGIN
                    Calculate(economy[j],i);
                    IF economy[j].emission/economy[j].income < minint THEN
                       minint:= economy[j].emission/economy[j].income
                END;
                CalcClimate(i);
                IF (i>=50) AND (i mod 10 = 0) THEN BEGIN
                   write(tunc,climate.temp:8:4,' ');
                   write(cunc,climate.acco2^[i]:10:4,' ');
                   totpop:= 0;
                   totinc:= 0;
                   totimp:= 0;
                   totemm:= 0;
                   totred:= 0;
                   FOR k:= 1 TO NoReg DO BEGIN
                       totpop:= totpop + economy[k].population;
                       totinc:= totinc + economy[k].income;
                       totemm:= totemm + economy[k].emission;
                       totimp:= totimp + (economy[k].eloss+economy[k].sloss)*economy[k].income;
                       totred:= totred + economy[k].taxmp*economy[k].ryg*economy[k].income;
                   END;
                   write(punc,totpop:10:4,' ');
                   write(gunc,totinc:10:4,' ');
                   write(eunc,totemm:10:4,' ');
(*                   write(iunc,totimp/totinc*100:8:4,' ');*)
                     write(iunc,economy[15].eloss+economy[15].sloss:8:4,' ');
                   write(runc,totred/totinc*100:10:8,' ')
                END
            END;
            writeln(tunc);
            writeln(cunc);
            writeln(punc);
            writeln(gunc);
            writeln(eunc);
            writeln(iunc);
            writeln(runc);
            FOR i:= 1 TO NoReg DO
                WITH Economy[i] DO BEGIN
                     NPVW:= hNPVW/run + (run-1)*NPVW/run;
                     write(wunc,hNPVW:20:4,' ')
                END;
            writeln(wunc);
            WITH climate DO BEGIN
                 c2300:= acco2^[NYear]/run + (run-1)*c2300/run;
                 c2275:= acco2^[NYear-25]/run + (run-1)*c2275/run;
                 c2250:= acco2^[NYear-50]/run + (run-1)*c2250/run;
                 c2225:= acco2^[NYear-75]/run + (run-1)*c2225/run;
                 c2200:= acco2^[NYear-100]/run + (run-1)*c2200/run;
                 c2175:= acco2^[NYear-125]/run + (run-1)*c2175/run;
                 c2150:= acco2^[NYear-150]/run + (run-1)*c2150/run;
                 c2125:= acco2^[NYear-175]/run + (run-1)*c2125/run;
                 c2100:= acco2^[NYear-200]/run + (run-1)*c2100/run;
                 c2075:= acco2^[NYear-225]/run + (run-1)*c2075/run;
                 c2050:= acco2^[NYear-250]/run + (run-1)*c2050/run;
                 c2025:= acco2^[NYear-275]/run + (run-1)*c2025/run;
                 c2000:= acco2^[NYear-300]/run + (run-1)*c2000/run
            END
        END;
        close(wunc);
        close(cunc);
        close(tunc);
        close(punc);
        close(eunc);
        close(gunc);
        close(iunc);
        close(runc);
     END
     ELSE BEGIN
(* once *)
          InitImpactPar;
          ReInit;
          assign(els,sessionpathname+'eloss.out');
          rewrite(els);
          assign(sls,sessionpathname+'sloss.out');
          rewrite(sls);
          assign(ent,sessionpathname+'enter.out');
          rewrite(ent);
          assign(mal,sessionpathname+'malaria.out');
          rewrite(mal);
          assign(sch,sessionpathname+'schisto.out');
          rewrite(sch);
          assign(den,sessionpathname+'dengue.out');
          rewrite(den);
          assign(car,sessionpathname+'cardio.out');
          rewrite(car);
          assign(res,sessionpathname+'respir.out');
          rewrite(res);
          assign(lea,sessionpathname+'leave.out');
          rewrite(lea);
          assign(inc,sessionpathname+'income.out');
          rewrite(inc);
          assign(wel,sessionpathname+'emissions.out');
          rewrite(wel);
          assign(sul,sessionpathname+'sulphur.out');
          rewrite(sul);
          assign(pop,sessionpathname+'population.out');
          rewrite(pop);
          assign(clim,sessionpathname+'climate.out');
          rewrite(clim);
          assign(mc,sessionpathname+'margcost.out');
          rewrite(mc);
          FOR i:= 1 TO NYear DO BEGIN
              write(els,i+1950,' ');
              write(sls,i+1950,' ');
              write(ent,i+1950,' ');
              write(mal,i+1950,' ');
              write(sch,i+1950,' ');
              write(den,i+1950,' ');
              write(car,i+1950,' ');
              write(res,i+1950,' ');
              write(lea,i+1950,' ');
              write(inc,i+1950,' ');
              write(wel,i+1950,' ');
              write(sul,i+1950,' ');
              write(pop,i+1950,' ');
              write(mc,i+1950,' ');
              totloss:= 0;
              maxkant:= 0;
              IF i < 40 THEN
                 minint:= 1000;
              FOR j:= 1 TO NoReg DO BEGIN
                  Calculate(economy[j],i);
                  loss:= (economy[j].eloss+economy[j].sloss);
                  IF loss > maxkant THEN
                     maxkant:= loss;
                  IF economy[j].emission/economy[j].income < minint THEN
                     minint:= economy[j].emission/economy[j].income;
                  Write1(economy[j],i,els,sls,ent,mal,sch,den,car,res,lea,inc,wel,sul,pop,mc);
                  totloss:= totloss + (economy[j].eloss+economy[j].sloss)*economy[j].income
              END;
              IF kantian THEN
                 FOR j:= 1 TO NoReg DO
                     WITH economy[j] DO BEGIN
                          kant:= maxkant - (eloss+sloss);
                          kant:= kant*income
                     END;
              IF compens THEN
                 FOR j:= 1 TO NoReg DO
                     WITH economy[j] DO BEGIN
                          income:= income*(1+eloss+sloss)-totloss*cummemm/globcum;
                          IF income < 0.0000001 THEN
                             income:= 0
                     END;
              IF aid AND (i>50) THEN BEGIN
                 transfer:= 0;
                 source:= 0;
                 FOR j:= 1 TO NoReg DO
                     WITH economy[j] DO
                          IF ypcold > ypc THEN
                             transfer:= transfer + (ypcold-ypc)*population/1000
                          ELSE
                              source:= source + 0.01*income;
                 IF (transfer > 0) AND (source > 0) AND (transfer < source) THEN
                    FOR j:= 1 TO NoReg DO
                        WITH economy[j] DO
                             IF ypcold > ypc THEN
                                income:= ypcold*population/1000
                             ELSE
                                 income:= income - income/source*transfer/1000
              END;
              CalcClimate(i);
              WITH climate DO BEGIN
                   write(clim,i+1950:4,' ',acco2^[i]:9:4,' ',acch4^[i]:9:4,' ',acn2o^[i]:9:4,' ',acsf6^[i]:9:4,' ');
                   write(clim,globso2:8:4,' ',radforc:8:4,' ',temp:11:8,' ',dt:10:8,' ',sea:6:3);
                   writeln(clim,cbox[1]:9:4,' ',cbox[2]:9:4,' ',cbox[3]:9:4,' ',cbox[4]:9:4,' ',cbox[5]:9:4);
              END;
              writeln(els);
              writeln(sls);
              writeln(ent);
              writeln(mal);
              writeln(sch);
              writeln(den);
              writeln(res);
              writeln(car);
              writeln(lea);
              writeln(inc);
              writeln(wel);
              writeln(sul);
              writeln(pop);
              writeln(mc)
          END;
          close(els);
          close(sls);
          close(ent);
          close(mal);
          close(sch);
          close(den);
          close(car);
          close(res);
          close(lea);
          close(inc);
          close(wel);
          close(sul);
          close(pop);
          close(mc);
          close(clim);
          FOR i:= 1 TO NoReg DO
              economy[i].NPVW:= economy[i].hNPVW;
(* twice *)
          InitImpactPar;
          ReInit;
          assign(sf6out,sessionpathname+'sf6.out');
          rewrite(sf6out);
          assign(ch4out,sessionpathname+'ch4.out');
          rewrite(ch4out);
          assign(n2oout,sessionpathname+'n2o.out');
          rewrite(n2oout);
          FOR i:= 1 TO NYear DO BEGIN
              write(sf6out,i+1950,' ');
              totloss:= 0;
              maxkant:= 0;
              IF i < 40 THEN
                 minint:= 1000;
              FOR j:= 1 TO NoReg DO BEGIN
                  Calculate(economy[j],i);
                  loss:= (economy[j].eloss+economy[j].sloss);
                  IF loss > maxkant THEN
                     maxkant:= loss;
                  IF economy[j].emission/economy[j].income < minint THEN
                     minint:= economy[j].emission/economy[j].income;
                  Write2(economy[j],i,ch4out,n2oout,sf6out);
                  totloss:= totloss + (economy[j].eloss+economy[j].sloss)*economy[j].income
              END;
              IF kantian THEN
                 FOR j:= 1 TO NoReg DO
                     WITH economy[j] DO BEGIN
                          kant:= maxkant - (eloss+sloss);
                          kant:= kant*income
                     END;
              IF compens THEN
                 FOR j:= 1 TO NoReg DO
                     WITH economy[j] DO BEGIN
                          income:= income*(1+eloss+sloss)-totloss*cummemm/globcum;
                          IF income < 0.0000001 THEN
                             income:= 0
                     END;
              IF aid AND (i>50) THEN BEGIN
                 transfer:= 0;
                 source:= 0;
                 FOR j:= 1 TO NoReg DO
                     WITH economy[j] DO
                          IF ypcold > ypc THEN
                             transfer:= transfer + (ypcold-ypc)*population/1000
                          ELSE
                              source:= source + 0.01*income;
                 IF (transfer > 0) AND (source > 0) AND (transfer < source) THEN
                    FOR j:= 1 TO NoReg DO
                        WITH economy[j] DO
                             IF ypcold > ypc THEN
                                income:= ypcold*population/1000
                             ELSE
                                 income:= income - income/source*transfer/1000
              END;
              CalcClimate(i);
(*              WITH climate DO BEGIN
                   write(clim,i+1950:4,' ',acco2^[i]:9:4,' ',acch4^[i]:9:4,' ',acn2o^[i]:9:4,' ',globso2:8:4,' ',radforc:8:4);
                   writeln(clim,' ',temp:11:8,' ',dt:10:8,' ',sea:6:3)
              END;*)
              writeln(sf6out);
              writeln(n2oout);
              writeln(ch4out);
          END;
          close(sf6out);
          close(n2oout);
          close(ch4out);
          (* close(clim) *)
          FOR i:= 1 TO NoReg DO
              economy[i].NPVW:= economy[i].hNPVW
     END
;writeln('exit runeconomy1');
END;

PROCEDURE MargRun1;
(* calculates the economic and climatic variables for the entire period *)

VAR i, j, k, run: word;
    els,sls,inc,pop,clim: text;
    totpop,totinc,totimp,totemm,totred,transfer,source: double;
    hp,hy,hw: regarr;

BEGIN
     assign(els,sessionpathname+'eloss.txt');
     rewrite(els);
     assign(sls,sessionpathname+'sloss.txt');
     rewrite(sls);
     assign(inc,sessionpathname+'income.txt');
     rewrite(inc);
     assign(pop,sessionpathname+'population.txt');
     rewrite(pop);
     assign(clim,sessionpathname+'climate.txt');
     rewrite(clim);
     FOR i:= 1 TO NYear DO BEGIN
         write(els,i+1950,' ');
         write(sls,i+1950,' ');
         write(inc,i+1950,' ');
         write(pop,i+1950,' ');
         write(clim,i+1950,' ');
         FOR j:= 1 TO NoReg DO BEGIN
             Calculate(economy[j],i);
             WriteM(economy[j],i,els,sls,inc,pop);
         END;
         IF aid AND (i>50) THEN BEGIN
            transfer:= 0;
            source:= 0;
            FOR j:= 1 TO NoReg DO
                WITH economy[j] DO
                     IF ypcold > ypc THEN
                        transfer:= transfer + (ypcold-ypc)*population/1000
                     ELSE
                         source:= source + 0.01*income;
(*            IF transfer > 0 THEN BEGIN
               write(i+1950:4,' ',transfer:10:2,' ',source:10:2);readln; END;*)
            IF (transfer > 0) AND (source > 0) AND (source > transfer) THEN
               FOR j:= 1 TO NoReg DO
                   WITH economy[j] DO
                        IF ypcold > ypc THEN
                           income:= ypcold*population/1000
                        ELSE
                            income:= income*(1 - 0.01*transfer/source)
         END;
         CalcClimate(i);
         write(clim,climate.temp:12:8,' ',climate.dt:12:8);
         writeln(els);
         writeln(sls);
         writeln(inc);
         writeln(pop);
         writeln(clim)
     END;
     close(els);
     close(sls);
     close(inc);
     close(pop);
     close(clim)
END;

PROCEDURE MargRun2;
(* calculates the economic and climatic variables for the entire period *)

VAR i, j, k, run: word;
    els,sls,inc,pop,clim: text;
    totpop,totinc,totimp,totemm,totred,source,transfer: double;
    hp,hy,hw: regarr;

BEGIN
     assign(els,sessionpathname+'eloss.out');
     rewrite(els);
     assign(sls,sessionpathname+'sloss.out');
     rewrite(sls);
     assign(inc,sessionpathname+'income.out');
     rewrite(inc);
     assign(pop,sessionpathname+'population.out');
     rewrite(pop);
     assign(clim,sessionpathname+'climate.out');
     rewrite(clim);
     FOR i:= 1 TO NYear DO BEGIN
         write(els,i+1950,' ');
         write(sls,i+1950,' ');
         write(inc,i+1950,' ');
         write(pop,i+1950,' ');
         write(clim,i+1950,' ');
         FOR j:= 1 TO NoReg DO BEGIN
             Calculate(economy[j],i);
             WriteM(economy[j],i,els,sls,inc,pop);
         END;
         IF aid AND (i>50) THEN BEGIN
            transfer:= 0;
            source:= 0;
            FOR j:= 1 TO NoReg DO
                WITH economy[j] DO
                     IF ypcold > ypc THEN
                        transfer:= transfer + (ypcold-ypc)*population/1000
                     ELSE
                         source:= source + 0.01*income;
(*            IF transfer > 0 THEN
               writeln(i+1950:4,' ',transfer:10:2,' ',source:10:2); *)
            IF (transfer > 0) AND (source > 0) AND (transfer < source) THEN
               FOR j:= 1 TO NoReg DO
                   WITH economy[j] DO
                        IF ypcold > ypc THEN
                           income:= ypcold*population/1000
                        ELSE
                            income:= income*(1 - 0.01*transfer/source)

         END;
         CalcClimate(i);
         write(clim,climate.temp:12:8,' ',climate.dt:12:8);
         writeln(els);
         writeln(sls);
         writeln(inc);
         writeln(pop);
         writeln(clim)
     END;
     close(els);
     close(sls);
     close(inc);
     close(pop);
     close(clim)
END;

PROCEDURE ShortRep(name: string);

VAR i: word;
    outf: text;

BEGIN
     assign(outf,sessionpathname+name);
     rewrite(outf);
     WriteLn(outf,'     tax   reg    forestry  region');
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              WriteLn(outf,tax[optper]:8:4,' ',forestry[optper]:8:2,'  ',name);
     WriteLn(outf);
     WriteLn(outf,'            NPVW        p        y        w       v    region');
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              WriteLn(outf,NPVW:16:4,' ',population:8:4,' ',income:8:4,' ',ypc:8:4,' ',name);
     WriteLn(outf);
     WriteLn(outf,'  income 90   region');
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              WriteLn(outf,GDP0:11:6,' ',name);
     WriteLn(outf);
     FOR i:= 28 TO NYear DO
         IF ((i+1970) MOD 25) = 0 THEN
            Writeln(outf,climate.acco2^[i]:12:6);
     close(outf)
END;

(* EXIT PROCEDURE *)

PROCEDURE Exit;

VAR i: word;

BEGIN
     FOR i:= 1 TO NoReg DO
         WITH Economy[i] DO BEGIN
              dispose(ypcgrowth);
              dispose(pgrowth);
              dispose(aeei);
              dispose(acei);
              dispose(ch4em);
              dispose(n2oem);
              dispose(forestemm);
              dispose(scentemp)
         END;
     WITH Climate DO BEGIN
          dispose(acco2);
          dispose(acch4);
          dispose(acn2o);
          dispose(acsf6)
     END
END;

(* MAIN PROCEDURES *)

PROCEDURE CalcMarg(VAR out, out2: text);

VAR els0,els1,sls0,sls1,inc0,pop0: text;
    oldinc,oldpop,newinc,newpop,elsbs,elspt,slsbs,slspt,df0,df1,df3,dfw,dfg,dfu,disc,md,npvmd: regarr;
    i,j: word;
    year,margreg0,margreg1,margreg3,margregw,margregg,margregu,margwld0,margwld1,margwld3,margwldw,margwldg,margwldu: double;
    wldpop, wldinc, greenbook: double;

(* dfw Weitzman discounting, dfg Gollier discounting, dfd UK Green Book discounting *)

BEGIN
     assign(els0,sessionpathname+'eloss.txt');
     reset(els0);
     assign(els1,sessionpathname+'eloss.out');
     reset(els1);
     assign(sls0,sessionpathname+'sloss.txt');
     reset(sls0);
     assign(sls1,sessionpathname+'sloss.out');
     reset(sls1);
     assign(inc0,sessionpathname+'income.txt');
     reset(inc0);
     assign(pop0,sessionpathname+'populati.txt');
     reset(pop0);
     read(inc0,year);
     read(pop0,year);
     read(els0,year);
     read(els1,year);
     read(sls0,year);
     read(sls1,year);
     FOR i:= 1 TO NoReg DO BEGIN
         read(inc0,oldinc[i]);
         read(pop0,oldpop[i]);
         read(els0,elsbs[i]);
         read(els1,elspt[i]);
         read(sls0,slsbs[i]);
         read(sls1,slspt[i]);
         df0[i]:= 1;
         df1[i]:= 1;
         df3[i]:= 1;
         dfw[i]:= 1;
	 dfg[i]:= 1;
	 dfu[i]:= 1;
         npvmd[i]:= 0
     END;
     margreg0:= 0;
     margreg1:= 0;
     margreg3:= 0;
     margregw:= 0;
     margregg:= 0;
     margregu:= 0;
     margwld0:= 0;
     margwld1:= 0;
     margwld3:= 0;
     margwldw:= 0;
     margwldg:= 0;
     margwldu:= 0;
     readln(inc0);
     readln(pop0);
     readln(els0);
     readln(els1);
     readln(sls0);
     readln(sls1);
     FOR i:= 2 TO NYear DO BEGIN
         read(inc0,year);
         read(pop0,year);
         read(els0,year);
         read(els1,year);
         read(sls0,year);
         read(sls1,year);
         wldpop:= 0;
         wldinc:= 0;
         FOR j:= 1 TO NoReg DO BEGIN
             read(inc0,newinc[j]);
             read(pop0,newpop[j]);
             wldpop:= wldpop + newpop[j];
             wldinc:= wldinc + newinc[j];
             read(els0,elsbs[j]);
             read(els1,elspt[j]);
             read(sls0,slsbs[j]);
             read(sls1,slspt[j])
         END;
         readln(inc0);
         readln(pop0);
         readln(els0);
         readln(els1);
         readln(sls0);
         readln(sls1);
         FOR j:= 1 TO NoReg DO BEGIN
             IF (newpop[j] < 0.00001) OR (newinc[j] < 0.00001) THEN
                disc[j]:= 0
             ELSE
                 disc[j]:= (newinc[j]/newpop[j]-oldinc[j]/oldpop[j])/(oldinc[j]/oldpop[j]);
         END;
         IF (i > 49) (*AND (i < 150)*) THEN BEGIN
            FOR j:= 1 TO NoReg DO BEGIN
                md[j]:= (elspt[j]+slspt[j]-elsbs[j]-slsbs[j])*newinc[j];
                npvmd[j]:= npvmd[j] + md[j]*df1[j];
                margreg0:= margreg0 + md[j]*df0[j];
                margreg1:= margreg1 + md[j]*df1[j];
                margreg3:= margreg3 + md[j]*df3[j];
                margregw:= margregw + md[j]*dfw[j];
                margregg:= margregg + md[j]*dfg[j];
                margregu:= margregu + md[j]*dfu[j];
                IF newinc[j] > 0.00001 THEN BEGIN
                   margwld0:= margwld0 + md[j]*df0[j]*wldinc*newpop[j]/wldpop/newinc[j];
                   margwld1:= margwld1 + md[j]*df1[j]*wldinc*newpop[j]/wldpop/newinc[j];
                   margwld3:= margwld3 + md[j]*df3[j]*wldinc*newpop[j]/wldpop/newinc[j];
                   margwldw:= margwldw + md[j]*dfw[j]*wldinc*newpop[j]/wldpop/newinc[j];
                   margwldg:= margwldg + md[j]*dfg[j]*wldinc*newpop[j]/wldpop/newinc[j];
                   margwldu:= margwldu + md[j]*dfu[j]*wldinc*newpop[j]/wldpop/newinc[j]
                END;
                df0[j]:= df0[j]/(1.00+disc[j]);
                df1[j]:= df1[j]/(1.01+disc[j]);
                df3[j]:= df3[j]/(1.03+disc[j]);
                dfg[j]:= dfg[j]/(1.00+(0.01*((1+49*pow(2.713,-0.06*(i-50)))/( (1+7*pow(2.713,-0.06*(i-50)))))) + disc[j]);
		dfg[j]:= 1;
                dfw[j]:= dfw[j]/(1+0.375/(12.5+i-50)+disc[j]);
                IF i<80 THEN
                  greenbook:=0.035
                ELSE IF i<125 THEN
                  greenbook:=0.03
                ELSE IF i<175 THEN
                  greenbook:=0.025
                ELSE IF i<250 THEN
                  greenbook:=0.02
                ELSE IF i<350 THEN
                  greenbook:=0.015
                ELSE
                  greenbook:=0.01;
                dfu[j]:= dfu[j]/(1.00 + greenbook);

            END
         END;
         FOR j:= 1 TO NoReg DO BEGIN
             oldinc[j]:= newinc[j];
             oldpop[j]:= newpop[j]
         END
     END;
     close(els0);
     close(els1);
     close(sls0);
     close(sls1);
     close(inc0);
     close(pop0);
     write(out,margreg0:14:8,' ',margwld0:14:8,' ');
     write(out,margreg1:14:8,' ',margwld1:14:8,' ');
     write(out,margreg3:14:8,' ',margwld3:14:8,' ');
     write(out,margregw:14:8,' ',margwldw:14:8,' ');
     write(out,margregg:14:8,' ',margwldg:14:8,' ');
     writeln(out,margregu:14:8,' ',margwldu:14:8);
     FOR i:= 1 TO NoReg DO
         write(out2,npvmd[i]:14:8,' ');
     writeln(out2)
END;

PROCEDURE Main;

BEGIN
     Initialise;
     InitImpactPar;
     ReInit;
     SetMeasures1;
     RunEconomy1;
     ShortRep('summary.out');
     Exit
END;

PROCEDURE ShortMain;

BEGIN
     Initialise;
     InitImpactPar;
     ReInit;
     RunEconomy;
     Exit
END;

PROCEDURE MargMain;

VAR i: word;
    out, out2: text;

BEGIN
     write('calculating marginal damage costs ');
     Initialise;
     InitImpactPar;
     assign(out,sessionpathname+'margunc.out');
     rewrite(out);
     assign(out2,sessionpathname+'regmarg.out');
     rewrite(out2);
     SetMeasures1;

(*     MCA[0]:=  0.0;
     MCA[1]:=  0.0;
     MCA[2]:=  0.0;
     MCA[3]:=  0.0;
     MCA[4]:=  0.0;
     MCA[5]:=  0.0;
     MCA[6]:=  0.0;
     MCA[7]:=  0.0;
     MCA[8]:=  0.0;
     MCA[9]:=  0.0;
     MCA[10]:= 0.0;*)

     FOR mdp:= 5 TO 14 DO BEGIN
         addmp:= 0;
         ReInit;
         MargRun1;
         addmp:= 1;
         ReInit;
         MargRun2;
         CalcMarg(out,out2)
     END;
     mdp:= 5;
(*     FOR i:= 1 TO 8490 DO
         DrawRandom;*)
     FOR i:= 1 TO Runs DO BEGIN
         write(i:4,' ');
         DrawRandom;
         ReInit;
         addmp:= 0;
         MargRun1;
         write('run1 ');
         addmp:= 1;
         ReInit;
         MargRun2;
         writeln('run2');
         CalcMarg(out,out2)
     END;
     close(out);
     close(out2);
     Exit
END;

PROCEDURE CBAMain;

VAR i, j, k: word;
    inp: text;
    share: ARRAY [1..10,1..NoReg] OF double;

BEGIN
     FOR i:= 0 TO 10 DO
         FOR j:= 1 TO NoReg DO
             MCA[i,j]:= 0;
(*     MargMain;
     assign(inp,sessionpathname+'shares.001');
     reset(inp);
     FOR i:= 1 TO 10 DO BEGIN
         FOR j:= 1 TO NoReg DO
             read(inp,share[i,j]);
         readln(inp)
     END;
     close(inp);
     FOR k:= 1 TO 10 DO BEGIN
         assign(inp,sessionpathname+'regmarg.out');
         reset(inp);
         FOR i:= 1 TO 10 DO BEGIN
             FOR j:= 1 TO NoReg DO
                 read(inp,MCA[i,j]);
             readln(inp)
         END;
         close(inp);
         FOR i:= 1 TO 10 DO
             MCA[i,0]:= 0;
         FOR i:= 1 TO 10 DO
             FOR j:= 1 TO NoReg DO
                 MCA[i,0]:= MCA[i,0]+MCA[i,j];
         IF cooperate THEN
            FOR i:= 1 TO 10 DO
                FOR j:= 1 TO NoReg DO
                    MCA[i,j]:= MCA[i,0];
         IF liability THEN
            FOR i:= 1 TO 10 DO
                FOR j:= 1 TO NoReg DO
                    MCA[i,j]:= share[i,j]*MCA[i,0];
         MargMain
     END;*)
(*     FOR i:= 1 TO NoReg DO
         MCA[1,i]:= 15.95;
     FOR i:= 1 TO NoReg DO
         MCA[2,i]:= 15.01;
     FOR i:= 1 TO NoReg DO
         MCA[3,i]:= 13.92;
     FOR i:= 1 TO NoReg DO
         MCA[4,i]:= 12.77;
     FOR i:= 1 TO NoReg DO
         MCA[5,i]:= 11.62;
     FOR i:= 1 TO NoReg DO
         MCA[6,i]:= 10.50;
     FOR i:= 1 TO NoReg DO
         MCA[7,i]:= 9.42;
     FOR i:= 1 TO NoReg DO
         MCA[8,i]:= 8.42;
     FOR i:= 1 TO NoReg DO
         MCA[9,i]:= 7.51;
     FOR i:= 1 TO NoReg DO
         MCA[10,i]:= 6.68;*)
     MargMain
END;

PROCEDURE MetaMain;

VAR i: word;
    out: text;

BEGIN
     SetMeasures1;
     assign(out,sessionpathname+'radforc.out');
     rewrite(out);
     MC:= 0.0;
     FOR i:= 0 TO 10000 DO BEGIN
         write(MC:6:1);
         MaxRF:= 0;
         MaxYR:= 1950;
         T2100:= 0;
         ShortMain;
         WriteLn(out,MC:6:1,' ',MaxRF:12:8,' ',MaxYr:4:0,' ',MaxPCO2:12:8,' ',RF2100:12:8,' ',PCO2100:12:8,' ',T2100:12:8);
         MC:= MC+0.1
     END;
     close(out);
END;

BEGIN
     mdp:= 5;
     MC:= 32.2;
     Main;
     WriteLn(MaxRF:8:4,' ',MaxYr:4:0,' ',RF2100:8:4,' ',RF2300:8:4);
END.